import os

from .test_tasktest import *
from wpiformat.eofnewline import EofNewline


def test_eofnewline():
    test = TaskTest(EofNewline())

    file_appendix = (
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + os.linesep
        + "int main() {"
        + os.linesep
        + '  std::cout << "Hello World!";'
        + os.linesep
        + "}"
    )

    # Empty file
    test.add_input("./Test.h", "")
    test.add_output("\n", True)

    test_output = file_appendix + os.linesep

    # No newline
    test.add_input("./Test.h", file_appendix)
    test.add_output(test_output, True)

    # One newline
    test.add_input("./Test.h", test_output)
    test.add_latest_input_as_output(True)

    # Two newlines
    test.add_input("./Test.h", test_output + os.linesep)
    test.add_output(test_output, True)

    # .bat file with no "./" prefix
    test.add_input("test.bat", file_appendix.replace(os.linesep, "\r\n"))
    test.add_output(test_output.replace(os.linesep, "\r\n"), True)

    test.run(OutputType.FILE)



from datetime import date
import subprocess


def get_version():
    proc = subprocess.run(
        [
            "git",
            "rev-list",
            "--count",
            # Includes previous year's commits in case one was merged after the
            # year incremented. Otherwise, the version wouldn't increment.
            '--after="main@{' + str(date.today().year - 1) + '-01-01}"',
            "main",
        ],
        check=True,
        encoding="utf-8",
        stdout=subprocess.PIPE,
    )
    # If there is no main branch, the commit count defaults to 0
    if proc.returncode:
        commit_count = "0"
    else:
        commit_count = proc.stdout.rstrip()

    # Version number: <year>.<# commits on main>
    return f"{date.today().year}.{commit_count}"



"""This task ensures GoogleTest test names follow the format
"TEST(ThingTest, Thing)".
"""

import regex

from wpiformat.task import Task


class GTestName(Task):
    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_cpp_file(name)

    def run_pipeline(self, config_file, name, lines):
        output = ""
        success = True

        test_name_rgx = regex.compile(
            r"\b(?P<test_type>TEST(_F|_P)?)\((?P<whitespace>\s*)(?P<test_suite>\w+), (?P<test_case>\w+)\)"
        )
        extract_location = 0
        for match in test_name_rgx.finditer(lines):
            test_type = match.group("test_type")
            test_suite = match.group("test_suite")
            test_case = match.group("test_case")

            # Write lines prior to test
            output += lines[extract_location : match.start()]

            # Write test type
            output += test_type + "("
            if match.group("whitespace"):
                output += match.group("whitespace")

            # Fix test suite name
            if test_suite.endswith("Tests"):
                test_suite = test_suite[:-1]
            if not test_suite.endswith("Test"):
                test_suite += "Test"

            # Write test suite name
            output += test_suite

            # Fix test case name
            if test_case == "Test" or test_case == "Tests":
                print(
                    f"Error: {name}: undescriptive test case name '{test_case}' in '{test_suite}.{test_case}'"
                )
                success = False
            else:
                if test_case.endswith("Test"):
                    test_case = test_case[:-4]
                if test_case.endswith("Tests"):
                    test_case = test_case[:-5]

            # Write test case name
            output += ", " + test_case + ")"

            extract_location = match.end()

        # If input has unprocessed lines, write them to output
        if extract_location < len(lines):
            output += lines[extract_location:]

        # Reset for next regex iteration
        lines = output
        output = ""

        inst_rgx = regex.compile(
            r"INSTANTIATE_TEST_SUITE_P\((?P<whitespace>\s*)(?P<test_suite>\w+), (?P<test_case>\w+)"
        )
        extract_location = 0
        for match in inst_rgx.finditer(lines):
            test_suite = match.group("test_suite")
            test_case = match.group("test_case")

            # Write lines prior to test
            output += lines[extract_location : match.start()]
            output += "INSTANTIATE_TEST_SUITE_P("
            if match.group("whitespace"):
                output += match.group("whitespace")

            # Fix test suite name
            if test_suite.endswith("Test"):
                test_suite += "s"
            if not test_suite.endswith("Tests"):
                test_suite += "Tests"

            # Write test suite name
            output += test_suite

            # Fix test case name
            if test_case.endswith("Tests"):
                test_case = test_case[:-1]
            if not test_case.endswith("Test"):
                test_case += "Test"

            # Write test case name
            output += ", " + test_case

            extract_location = match.end()

        # If input has unprocessed lines, write them to output
        if extract_location < len(lines):
            output += lines[extract_location:]

        return output, success



from datetime import date
import os
from pathlib import Path
import shutil
import subprocess
import tempfile

from .test_tasktest import *
from wpiformat.config import Config
from wpiformat.licenseupdate import LicenseUpdate


class OpenTemporaryDirectory:
    def __init__(self):
        self.prev_dir = os.getcwd()

    def __enter__(self):
        self.temp_dir = tempfile.TemporaryDirectory()
        os.chdir(self.temp_dir.name)
        return self.temp_dir

    def __exit__(self, type, value, traceback):
        os.chdir(self.prev_dir)


def test_licenseupdate():
    year = str(date.today().year)

    task = LicenseUpdate()
    test = TaskTest(task)

    file_appendix = (
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + os.linesep
        + "int main() {"
        + os.linesep
        + '  std::cout << "Hello World!";'
        + os.linesep
        + "}"
    )

    # pragma once at top of file
    test.add_input("./Test.h", file_appendix)
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # pragma once at top of file preceded by newline
    test.add_input("./Test.h", os.linesep + file_appendix)
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # File containing up-to-date license preceded by newline
    test.add_input(
        "./Test.h",
        os.linesep
        + "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # File containing up-to-date range license
    test.add_input(
        "./Test.h",
        "/*                                Company Name                                */"
        + os.linesep
        + "// Copyright (c) 2011-{} Company Name. All Rights Reserved.".format(year)
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) 2011-{} Company Name. All Rights Reserved.                 */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # File containing up-to-date license with one year
    test.add_input(
        "./Test.h",
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_latest_input_as_output(True)

    # File with three newlines between license and include guard
    test.add_input(
        "./Test.h",
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # File with only one newline between license and include guard
    test.add_input(
        "./Test.h",
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + file_appendix,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # File with multiline comment spanning multiple lines of license header
    test.add_input(
        "./Test.h",
        "/*"
        + os.linesep
        + " * Autogenerated file! Do not manually edit this file. This version is regenerated"
        + os.linesep
        + " * any time the publish task is run, or when this file is deleted."
        + os.linesep
        + " */"
        + os.linesep
        + os.linesep
        + 'const char* WPILibVersion = "";',
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) {} Company Name. All Rights Reserved.                      */".format(
            year
        )
        + os.linesep
        + os.linesep
        + "/*"
        + os.linesep
        + " * Autogenerated file! Do not manually edit this file. This version is regenerated"
        + os.linesep
        + " * any time the publish task is run, or when this file is deleted."
        + os.linesep
        + " */"
        + os.linesep
        + os.linesep
        + 'const char* WPILibVersion = "";',
        True,
    )

    # File containing license year range in different postion than template
    # (If the year isn't extracted, the range will be replaced with one year and
    # the test will fail.)
    test.add_input(
        "./Test.h",
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) Company Name 2011-{}. All Rights Reserved.                 */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) 2011-{} Company Name. All Rights Reserved.                 */".format(
            year
        )
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # Ensure "/*" after "*/" on same line is detected
    test.add_input(
        "./Test.h",
        "/*----------------------------------------------------------------------------*/"
        + os.linesep
        + "/* Copyright (c) 2011 FIRST. All Rights Reserved.                             */"
        + os.linesep
        + "/* Open Source Software - may be modified and shared by FRC teams. The code   */"
        + os.linesep
        + "/* must be accompanied by the FIRST BSD license file in the root directory of */"
        + os.linesep
        + "/* the project.                                                               */"
        + os.linesep
        + "/*----------------------------------------------------------------------------*//*"
        + os.linesep
        + os.linesep
        + "blah"
        + os.linesep
        + os.linesep
        + "*/"
        + os.linesep,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) 2011-{} Company Name. All Rights Reserved.                 */".format(
            year
        )
        + os.linesep
        + os.linesep,
        True,
    )

    # File excluded from license update isn't modified
    test.add_input(
        "./Excluded.h",
        "/* Copyright (c) Company Name 2011-{}. */".format(year)
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_latest_input_as_output(True)

    # Ensure license regex matches
    test.add_input(
        "./Test.h",
        "/* Company Name */"
        + os.linesep
        + "/* Copyright (c) 1992-2015 Company Name. All Rights Reserved. */"
        + os.linesep
        + os.linesep
        + file_appendix,
    )
    test.add_output(
        "/*                                Company Name                                */"
        + os.linesep
        + "/* Copyright (c) 1992-"
        + year
        + " Company Name. All Rights Reserved.                 */"
        + os.linesep
        + os.linesep
        + file_appendix,
        True,
    )

    # Ensure excluded files won't be processed
    config_file = Config(os.path.abspath(os.getcwd()), ".styleguide")
    assert not task.should_process_file(config_file, "./Excluded.h")

    # Create git repo to test license years for commits
    with OpenTemporaryDirectory():
        subprocess.run(["git", "init", "-q"])

        # Add base files
        with open(".styleguide-license", "w") as file:
            file.write("// Copyright (c) {year}")
        with open(".styleguide", "w") as file:
            file.write("cppSrcFileInclude {\n" + r"\.cpp$")
        subprocess.run(["git", "add", ".styleguide-license"])
        subprocess.run(["git", "add", ".styleguide"])
        subprocess.run(["git", "commit", "-q", "-m", '"Initial commit"'])

        # Add file with commit date of last year and range through this year
        with open("last-year.cpp", "w") as file:
            file.write(f"// Copyright (c) 2017-{year}")
        subprocess.run(["git", "add", "last-year.cpp"])
        subprocess.run(["git", "commit", "-q", "-m", '"Last year"'])
        last_iso_year = f"{int(year) - 1}-01-01T00:00:00"
        subprocess.Popen(
            ["git", "commit", "-q", "--amend", "--no-edit", f"--date={last_iso_year}"],
            env={**os.environ, "GIT_COMMITTER_DATE": last_iso_year},
        ).wait()

        # Add file with commit date of this year and range through this year
        with open("this-year.cpp", "w") as file:
            file.write(f"// Copyright (c) 2017-{year}")
        subprocess.run(["git", "add", "this-year.cpp"])
        subprocess.run(["git", "commit", "-q", "-m", '"This year"'])

        # Add file with commit date of next year and range through this year
        with open("next-year.cpp", "w") as file:
            file.write(f"// Copyright (c) 2017-{year}")
        subprocess.run(["git", "add", "next-year.cpp"])
        subprocess.run(["git", "commit", "-q", "-m", '"Next year"'])
        next_iso_year = f"{int(year) + 1}-01-01T00:00:00"
        subprocess.Popen(
            ["git", "commit", "-q", "--amend", "--no-edit", f"--date={next_iso_year}"],
            env={**os.environ, "GIT_COMMITTER_DATE": next_iso_year},
        ).wait()

        # Create uncommitted file with no year
        Path("no-year.cpp").touch()

        # Run wpiformat on last-year.cpp
        with open("last-year.cpp", "r") as input:
            lines = input.read()
        output, success = task.run_pipeline(config_file, "last-year.cpp", lines)
        assert output == f"// Copyright (c) 2017-{int(year) - 1}\n\n"

        # Run wpiformat on last-year.cpp with uncommitted changes. It should
        # update to next year instead of keeping previous year
        with open("last-year.cpp", "a") as input:
            input.write("change\n")
        output, success = task.run_pipeline(
            config_file, "last-year.cpp", lines + "change\n"
        )
        assert output == f"// Copyright (c) 2017-{year}\n\nchange\n"

        # Erase changes made to last-year.cpp in previous test
        with open("last-year.cpp", "w") as input:
            input.write(lines)

        # Run wpiformat on this-year.cpp
        with open("last-year.cpp", "r") as input:
            lines = input.read()
        output, success = task.run_pipeline(config_file, "this-year.cpp", lines)
        assert output == f"// Copyright (c) 2017-{year}\n\n"

        # Run wpiformat on next-year.cpp
        with open("next-year.cpp", "r") as input:
            lines = input.read()
        output, success = task.run_pipeline(config_file, "next-year.cpp", lines)
        assert output == f"// Copyright (c) 2017-{int(year) + 1}\n\n"

        # Run wpiformat on no-year.cpp
        # Should have current calendar year
        with open("no-year.cpp", "r") as input:
            lines = input.read()
        output, success = task.run_pipeline(config_file, "no-year.cpp", lines)
        assert output == f"// Copyright (c) {year}\n\n"

    test.run(OutputType.FILE)



"""This class is for handling wpiformat config files."""

import os
import regex
import sys


class Config:
    def __init__(self, directory, file_name):
        """Constructor for Config object.

        Keyword arguments:
        directory -- directory in which to start search for file
        file_name -- file name string
        """
        self.__config_dict = self.__parse_config_file(directory, file_name)
        self.__c_header_include_regex = self.regex("cHeaderFileInclude")
        self.__cpp_header_include_regex = self.regex("cppHeaderFileInclude")
        self.__cpp_src_include_regex = self.regex("cppSrcFileInclude")
        self.__generated_exclude_regex = self.regex("generatedFileExclude")
        self.__modifiable_exclude_regex = self.regex("modifiableFileExclude")

    @staticmethod
    def read_file(directory, file_name):
        """Find file and return contents.

        Checks current directory for file. If one doesn't exist, try all parent
        directories as well.

        Keyword arguments:
        directory -- current directory from which to start search
        file_name -- file name string

        Returns tuple of file name and list containing file contents or triggers
        program exit.
        """
        file_found = False
        while not file_found:
            try:
                with open(directory + os.sep + file_name, "r") as file_contents:
                    file_found = True
                    return (
                        os.path.join(directory, file_name),
                        file_contents.read().splitlines(),
                    )
            except OSError:
                # .git files are ignored, which are created within submodules
                if os.path.isdir(directory + os.sep + ".git"):
                    print(
                        "Error: config file '"
                        + file_name
                        + "' not found in '"
                        + directory
                        + "'"
                    )
                    sys.exit(1)
                directory = os.path.dirname(directory)

    def group(self, group_name):
        """Returns value from config dictionary given key string.

        Keyword arguments:
        group_name -- config group name
        """
        if not self.__config_dict:
            return []

        try:
            return self.__config_dict[group_name]
        except KeyError:
            return []

    def regex(self, *args):
        """Converts contents of group from config file into regex.

        Keyword arguments:
        *args -- argument list of groups. They are all joined by "|".

        Returns compiled regex.
        """
        group_contents = []

        for group_name in args:
            group = self.group(group_name)

            # If group exists
            if len(group) > 0:
                group_contents.extend(group)

        if len(group_contents) == 0:
            # If regex string is empty, make regex match nothing
            return regex.compile(r"a^")
        else:
            return regex.compile(r"|".join(group_contents))

    def is_c_file(self, name):
        """Returns True if file is either C header or C source file.

        Keyword arguments:
        name -- file name string
        """
        return self.is_c_header_file(name) or self.is_c_src_file(name)

    def is_c_header_file(self, name):
        """Returns True if file is C header file.

        Keyword arguments:
        name -- file name string
        """
        return self.__c_header_include_regex.search(name) is not None

    @staticmethod
    def is_c_src_file(name):
        """Returns True if file is C source file.

        Keyword arguments:
        name -- file name string
        """
        return name.endswith(".c")

    def is_cpp_file(self, name):
        """Returns True if file is either C++ header or C++ source file.

        Keyword arguments:
        name -- file name string
        """
        return self.is_cpp_header_file(name) or self.is_cpp_src_file(name)

    def is_cpp_header_file(self, name):
        """Returns True if file is C++ header file.

        Keyword arguments:
        name -- file name string
        """
        return self.__cpp_header_include_regex.search(name) is not None

    def is_cpp_src_file(self, name):
        """Returns True if file is C++ source file.

        Keyword arguments:
        name -- file name string
        """
        return self.__cpp_src_include_regex.search(name) is not None

    def is_header_file(self, name):
        """Returns True if file is either C or C++ header file.

        Keyword arguments:
        name -- file name string
        """
        return self.is_c_header_file(name) or self.is_cpp_header_file(name)

    def is_generated_file(self, name):
        """Returns True if file is generated (generated files are skipped).

        Keyword arguments:
        name -- file name string
        """
        return self.__generated_exclude_regex.search(name) is not None

    def is_modifiable_file(self, name):
        """Returns True if file is modifiable but should be skipped.

        Keyword arguments:
        name -- file name string
        """
        return self.__modifiable_exclude_regex.search(name) is not None

    def __parse_config_file(self, directory, file_name):
        """Parse values from config file.

        Checks current directory for config file. If one doesn't exist, try all
        parent directories as well.

        Keyword arguments:
        directory -- current directory from which to start search
        file_name -- config file name string

        Returns dictionary of groups (group name -> list of values).
        """
        in_group = False
        config_group = {}
        group_name = ""
        group_elements = []

        self.file_name, lines = self.read_file(directory, file_name)
        if not lines:
            return None

        for line in lines:
            # Skip empty lines
            if line.strip() == "":
                continue

            if "{" in line:
                in_group = True

                # Group name is on same line as "{"
                group_name = line[: line.find("{")].strip()
            elif "}" in line:
                in_group = False

                # After group closes, save element list and clear it
                config_group[group_name] = group_elements
                group_elements = []
            elif in_group:
                value = line.strip()

                # Header includes still use forward slash on Windows
                unescaped_groups = [
                    "includeRelated",
                    "includeCSys",
                    "includeCppSys",
                    "includeOtherLibs",
                    "includeProject",
                ]
                if group_name == "includeGuardRoots":
                    # Include guard roots use native directory separators
                    value = value.replace("/", os.sep)
                elif group_name not in unescaped_groups and os.sep == "\\":
                    # Replace "/" with escaped "\" for regexes on Windows
                    value = value.replace("/", os.sep + os.sep)

                group_elements.append(value)
        return config_group



"""This task ensures that the file has exactly one EOF newline."""

from wpiformat.task import Task


class EofNewline(Task):
    def run_pipeline(self, config_file, name, lines):
        return lines.rstrip() + super().get_linesep(lines), True



import os

from .test_tasktest import *
from wpiformat.usingnamespacestd import UsingNamespaceStd


def test_usingnamespacestd():
    test = TaskTest(UsingNamespaceStd())

    warning_str = 'avoid "using namespace std;" in production software. While it is used in introductory C++, it pollutes the global namespace with standard library symbols. Be more specific and use "using std::thing;" instead.\n'

    # Hello World
    test.add_input(
        "./Main.cpp",
        "using namespace std;"
        + os.linesep
        + os.linesep
        + "int main() {"
        + os.linesep
        + '  cout << "Hello World!"'
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("Warning: ./Main.cpp: 1: " + warning_str, True)

    # Inside braces and not first line
    test.add_input(
        "./Main.cpp",
        "int main() {"
        + os.linesep
        + "  using namespace std;"
        + os.linesep
        + '  cout << "Hello World!"'
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("Warning: ./Main.cpp: 2: " + warning_str, True)

    # std::chrono
    test.add_input(
        "./Main.cpp",
        "#include <thread>"
        + os.linesep
        + os.linesep
        + "int main() {"
        + os.linesep
        + "  using namespace std::chrono;"
        + os.linesep
        + os.linesep
        + "  std::this_thread::sleep_for(10ms);"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("Warning: ./Main.cpp: 4: " + warning_str, True)

    # Ignore std::literals
    test.add_input("./Main.cpp", "using namespace std::literals;" + os.linesep)
    test.add_output("", True)

    # Ignore std::chrono_literals
    test.add_input("./Main.cpp", "using namespace std::chrono_literals;" + os.linesep)
    test.add_output("", True)

    # Ignore std::string_view_literals
    test.add_input(
        "./Main.cpp", "using namespace std::string_view_literals;" + os.linesep
    )
    test.add_output("", True)

    # Ignore std::placeholders
    test.add_input("./Main.cpp", "using namespace std::placeholders;" + os.linesep)
    test.add_output("", True)

    test.run(OutputType.STDOUT)



"""This task replaces empty C identifier lists "()" with "(void)"."""

import regex

from wpiformat.task import Task


class CIdentList(Task):
    @staticmethod
    def __print_failure(name):
        print(
            "Error: " + name + ": unmatched curly braces when scanning for "
            "C identifier lists. If the code compiles, this is a bug in "
            "wpiformat."
        )

    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_c_file(name) or config_file.is_cpp_file(name)

    def run_pipeline(self, config_file, name, lines):
        linesep = super().get_linesep(lines)

        output = ""
        pos = 0

        # C files use C linkage by default
        is_c = config_file.is_c_file(name)

        # Tokenize as extern "C" or extern "C++" with optional {, open brace,
        # close brace, or () folllowed by { to disambiguate function calls.
        # extern is first to try matching a brace to it before classifying the
        # brace as generic.
        #
        # Valid function prototypes and definitions have return type, spaces,
        # function name, optional spaces, then braces. They are followed by ; or
        # {.
        #
        # "def\\s+\w+" matches preprocessor directives "#ifdef" and "#ifndef" so
        # their contents aren't used as a return type.
        #
        # "\w+\**\s+\w+\s*" matches a function return type with 0 or more
        # trailing asterisks, spaces, a function name, then spaces before the
        # open parenthesis
        preproc_str = r"#else|#endif|"
        comment_str = r"/\*|\*/|//|" + linesep + r"|"
        string_str = r"\\\\|\\\"|\"|"
        char_str = r"\\'|'|"
        extern_str = r"(?P<ext_decl>extern \"C(\+\+)?\")\s+(?P<ext_brace>\{)?|"
        braces_str = r"\{|\}|;|def\s+\w+|\w+\**\s+\w+\s*(?P<paren>\(\))"
        postfix_str = r"(?=\s*(;|\{))"
        token_regex = regex.compile(
            preproc_str
            + comment_str
            + string_str
            + char_str
            + extern_str
            + braces_str
            + postfix_str
        )

        EXTRA_POP_OFFSET = 2

        # If value is greater than pop offset, the value needs to be restored in
        # addition to an extra stack pop being performed. The pop offset is
        # removed before assigning to is_c.
        #
        # is_c + pop offset == 2: C lang restore that needs extra brace pop
        # is_c + pop offset == 3: C++ lang restore that needs extra brace pop
        extern_brace_indices = [is_c]

        in_preproc_else = False
        in_multicomment = False
        in_singlecomment = False
        in_string = False
        in_char = False
        for match in token_regex.finditer(lines):
            token = match.group()

            # Skip #else to #endif in case they have braces in them. This
            # assumes preprocessor directives are only used for conditional
            # compilation for different platforms and have the same amount of
            # braces in both branches. Nested preprocessor directives are also
            # not handled.
            if token == "#else":
                in_preproc_else = True
            elif token == "#endif":
                in_preproc_else = False
            if in_preproc_else:
                continue

            if token == "/*":
                if not in_singlecomment and not in_string and not in_char:
                    in_multicomment = True
            elif token == "*/":
                if not in_singlecomment and not in_string and not in_char:
                    in_multicomment = False
            elif token == "//":
                if not in_multicomment and not in_string and not in_char:
                    in_singlecomment = True
            elif in_singlecomment and linesep in token:
                # Ignore token if it's in a singleline comment. Only check it
                # for newlines to end the comment.
                in_singlecomment = False
            elif in_multicomment or in_singlecomment:
                # Tokens processed after this branch are ignored if they are in
                # comments
                continue
            elif token == '\\"':
                continue
            elif token == '"':
                if not in_char:
                    in_string = not in_string
            elif token == "\\'":
                continue
            elif token == "'":
                if not in_string:
                    in_char = not in_char
            elif in_string or in_char:
                # Tokens processed after this branch are ignored if they are in
                # double or single quotes
                continue
            elif token == "{":
                extern_brace_indices.append(is_c)
            elif token == "}":
                is_c = extern_brace_indices.pop()

                if len(extern_brace_indices) == 0:
                    self.__print_failure(name)
                    return lines, False

                # If the next stack frame is from an extern without braces, pop
                # it.
                if extern_brace_indices[-1] >= EXTRA_POP_OFFSET:
                    is_c = extern_brace_indices[-1] - EXTRA_POP_OFFSET
                    extern_brace_indices.pop()
            elif token == ";":
                if len(extern_brace_indices) == 0:
                    self.__print_failure(name)
                    return lines, False

                # If the next stack frame is from an extern without braces, pop
                # it.
                if extern_brace_indices[-1] >= EXTRA_POP_OFFSET:
                    is_c = extern_brace_indices[-1] - EXTRA_POP_OFFSET
                    extern_brace_indices.pop()
            elif token.startswith("extern"):
                # Back up language setting first
                if match.group("ext_brace"):
                    extern_brace_indices.append(is_c)
                else:
                    # Handling an extern without braces changing the language
                    # type is done by treating it as a pseudo-brace that gets
                    # popped as well when the next "}" or ";" is encountered.
                    # The "extra pop" offset is used as a flag on the top stack
                    # value that is checked whenever a pop is performed.
                    extern_brace_indices.append(is_c + EXTRA_POP_OFFSET)

                # Change language based on extern declaration
                if match.group("ext_decl") == 'extern "C"':
                    is_c = True
                else:
                    is_c = False
            elif match.group("paren") and "return " not in match.group() and is_c:
                # Replaces () with (void)
                output += lines[pos : match.span("paren")[0]] + "(void)"
                pos = match.span("paren")[0] + len("()")

        # Write rest of file if it wasn't all processed
        if pos < len(lines):
            output += lines[pos:]

        # Invariant: extern_brace_indices has one entry
        success = len(extern_brace_indices) == 1
        if not success:
            self.__print_failure(name)

        return output, success



import os

from .test_tasktest import *
from wpiformat.stdlib import Stdlib


def test_stdlib():
    test = TaskTest(Stdlib())

    test.add_input(
        "./Main.cpp",
        "#include <cstdint>"
        + os.linesep
        + "#include <stdlib.h>"
        + os.linesep
        + os.linesep
        + "int main() {"
        + os.linesep
        + "  auto mem = static_cast<std::uint8_t*>(malloc(5));"
        + os.linesep
        + "  std::int32_t i = 4;"
        + os.linesep
        + "  int32_t a = -2;"
        + os.linesep
        + "  std::uint_fast16_t j = 5;"
        + os.linesep
        + "  std::uint_fast16_t* k = &j;"
        + os.linesep
        + "  std::uint_fast16_t** l = &k;"
        + os.linesep
        + "  std::uint_fast16_t ** m = l;"
        + os.linesep
        + "  free(mem);"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "#include <stdint.h>"
        + os.linesep
        + "#include <cstdlib>"
        + os.linesep
        + os.linesep
        + "int main() {"
        + os.linesep
        + "  auto mem = static_cast<uint8_t*>(std::malloc(5));"
        + os.linesep
        + "  int32_t i = 4;"
        + os.linesep
        + "  int32_t a = -2;"
        + os.linesep
        + "  uint_fast16_t j = 5;"
        + os.linesep
        + "  uint_fast16_t* k = &j;"
        + os.linesep
        + "  uint_fast16_t** l = &k;"
        + os.linesep
        + "  uint_fast16_t ** m = l;"
        + os.linesep
        + "  std::free(mem);"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Test NOLINT on #include
    test.add_input(
        "./Main.cpp",
        "#include <cstdint>  // NOLINT"
        + os.linesep
        + "#include <stdlib.h>  // NOLINT"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Test NOLINT on function
    test.add_input(
        "./Main.cpp",
        "  abs()  // NOLINT"
        + os.linesep
        + "  abs()"
        + os.linesep
        + "  abs()  // NOLINT"
        + os.linesep,
    )
    test.add_output(
        "  abs()  // NOLINT"
        + os.linesep
        + "  std::abs()"
        + os.linesep
        + "  abs()  // NOLINT"
        + os.linesep,
        True,
    )

    # FILE should be recognized as type here
    test.add_input("./Class.cpp", "static FILE* Class::file = nullptr;")
    test.add_output("static std::FILE* Class::file = nullptr;", True)

    # FILE should not be recognized as type here
    test.add_input(
        "./Class.cpp",
        "static int Class::error1 = ERR_FILE;"
        + os.linesep
        + "#define FILE_LOG(level)"
        + os.linesep
        + "if (level > FILELog::ReportingLevel())"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Remove "std::" from beginning of line
    test.add_input(
        "./Class.cpp",
        "/**"
        + os.linesep
        + " *"
        + os.linesep
        + " */"
        + os.linesep
        + "std::size_t SizeUleb128(uint64_t val) {"
        + os.linesep
        + "  uint32_t result = 0;"
        + os.linesep,
    )
    test.add_output(
        "/**"
        + os.linesep
        + " *"
        + os.linesep
        + " */"
        + os.linesep
        + "size_t SizeUleb128(uint64_t val) {"
        + os.linesep
        + "  uint32_t result = 0;"
        + os.linesep,
        True,
    )

    # Remove "std::" from beginning of file
    test.add_input(
        "./Class.cpp",
        "std::size_t SizeUleb128(uint64_t val) {"
        + os.linesep
        + "  uint32_t result = 0;"
        + os.linesep,
    )
    test.add_output(
        "size_t SizeUleb128(uint64_t val) {"
        + os.linesep
        + "  uint32_t result = 0;"
        + os.linesep,
        True,
    )

    # Don't prepend "std::" to function name if it's a function definition
    # "time()" in other contexts is a C standard library function.
    test.add_input(
        "./Test.cpp", "uint64_t time() const { return m_val.last_change; }" + os.linesep
    )
    test.add_latest_input_as_output(True)

    # Test detection of type within static_cast<>
    test.add_input("./Test.cpp", "static_cast<std::uint64_t>(x);" + os.linesep)
    test.add_output("static_cast<uint64_t>(x);" + os.linesep, True)

    # Test detection of type with "-" prefix
    test.add_input("./Test.cpp", "-atan(140 / 76.44);" + os.linesep)
    test.add_output("-std::atan(140 / 76.44);" + os.linesep, True)

    # Types followed by semicolon should match
    test.add_input("./Main.cpp", "typedef integer std::uint8_t;")
    test.add_output("typedef integer uint8_t;", True)

    # Remove "std::" from variadic template argument
    test.add_input(
        "./Class.cpp",
        "template <typename Char, typename Traits, std::size_t N1, std::size_t... N>"
        + os.linesep,
    )
    test.add_output(
        "template <typename Char, typename Traits, size_t N1, size_t... N>"
        + os.linesep,
        True,
    )

    test.run(OutputType.FILE)



import os

from .test_tasktest import *
from wpiformat.javaclass import JavaClass


def test_javaclass():
    test = TaskTest(JavaClass())

    # No line separators at beginning of class
    test.add_input(
        "./Test.java", "public class ExampleCommand extends Command {}" + os.linesep
    )
    test.add_latest_input_as_output(True)

    # One line separator at beginning of class
    test.add_input(
        "./Test.java",
        "public class ExampleCommand extends Command {"
        + os.linesep
        + "  public ExampleCommand() {}"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Two line separators at beginning of class
    test.add_input(
        "./Test.java",
        "public class ExampleCommand extends Command {"
        + os.linesep
        + os.linesep
        + "  public ExampleCommand() {}"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "public class ExampleCommand extends Command {"
        + os.linesep
        + "  public ExampleCommand() {}"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Three line separators at beginning of class
    test.add_input(
        "./Test.java",
        "public class ExampleCommand extends Command {"
        + os.linesep
        + os.linesep
        + os.linesep
        + "  public ExampleCommand() {}"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "public class ExampleCommand extends Command {"
        + os.linesep
        + "  public ExampleCommand() {}"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # class keyword in preceding comment to ensure regex matching it doesn't
    # continue past end of comment
    test.add_input(
        "./Test.java",
        "import edu.wpi.first.networktables.NetworkTableEntry;"
        + os.linesep
        + "import edu.wpi.first.wpilibj.Sendable;"
        + os.linesep
        + os.linesep
        + "/**"
        + os.linesep
        + " * A helper class for Shuffleboard containers to handle common child operations."
        + os.linesep
        + " */"
        + os.linesep
        + "final class ContainerHelper {"
        + os.linesep
        + os.linesep
        + "  private final ShuffleboardContainer m_container;"
        + os.linesep,
    )
    test.add_output(
        "import edu.wpi.first.networktables.NetworkTableEntry;"
        + os.linesep
        + "import edu.wpi.first.wpilibj.Sendable;"
        + os.linesep
        + os.linesep
        + "/**"
        + os.linesep
        + " * A helper class for Shuffleboard containers to handle common child operations."
        + os.linesep
        + " */"
        + os.linesep
        + "final class ContainerHelper {"
        + os.linesep
        + "  private final ShuffleboardContainer m_container;"
        + os.linesep,
        True,
    )

    test.run(OutputType.FILE)



"""This task warns against "using namespace std;"."""

import regex

from wpiformat.task import Task


class UsingNamespaceStd(Task):
    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_cpp_file(name)

    def run_pipeline(self, config_file, name, lines):
        linesep = super().get_linesep(lines)

        # Find instances of "using namespace std;" or subnamespaces of "std",
        # but not std::literals or std::chrono_literals.
        using_regex = regex.compile(
            r"using\s+namespace\s+std(;|::(?!(chrono_|string_view_)?literals|placeholders))"
        )

        for match in using_regex.finditer(lines):
            linenum = lines.count(linesep, 0, match.start()) + 1
            print(
                "Warning: "
                + name
                + ": "
                + str(linenum)
                + ': avoid "using namespace std;" in production software. While it is used in introductory C++, it pollutes the global namespace with standard library symbols. Be more specific and use "using std::thing;" instead.'
            )

        return lines, True



"""This task runs clang-format on the file."""

from subprocess import Popen, PIPE
import sys

import clang_format

from wpiformat.task import Task


class ClangFormat(Task):
    def __init__(self, clang_version):
        """Constructor for ClangFormat task.

        Keyword arguments:
        clang_version -- version number of clang-format appended to executable
                         name (deprecated for removal)
        """
        super().__init__()

        if clang_version == "":
            self.exec_name = clang_format._get_executable("clang-format")
        else:
            self.exec_name = "clang-format-" + clang_version

    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_c_file(name) or config_file.is_cpp_file(name)

    def run_pipeline(self, config_file, name, lines):
        try:
            p = Popen(
                [self.exec_name, "-style=file", "-assume-filename=" + name, "-"],
                stdin=PIPE,
                stdout=PIPE,
                encoding="utf-8",
            )
            stdout, stderr = p.communicate(input=lines)

            if p.returncode != 0:
                print(
                    f"error: {self.exec_name} returned non-zero exit status {p.returncode}",
                    file=sys.stderr,
                )
                print(stderr, file=sys.stderr)
                return lines, False
        except FileNotFoundError:
            print(
                f"error: {self.exec_name} not found in PATH. Is it installed?",
                file=sys.stderr,
            )
            return lines, False

        return stdout, True



"""This task ensures the trailing comment on the closing brace of extern and
namespace declarations matches that of the declaration name.
"""

import regex

from wpiformat.task import Task


class BraceComment(Task):
    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_c_file(name) or config_file.is_cpp_file(name)

    def run_pipeline(self, config_file, name, lines):
        linesep = super().get_linesep(lines)
        output = ""

        brace_prefix = r"(?P<prefix>(extern|namespace)\s+[\w\"]*)"
        brace_postfix = r"[ \t]*/(/|\*)[^\r\n]*"

        brace_regex = regex.compile(
            r"/\*|\*/|//|\\\\|\\\"|\"|\\'|'|"
            + linesep
            + r"|"
            + r"("
            + brace_prefix
            + r"\s*)?{|"  # "{" with optional prefix
            r"}(" + brace_postfix + r")?"
        )  # "}" with optional comment postfix

        name_stack = []
        brace_count = 0
        extract_location = 0
        in_multicomment = False
        in_singlecomment = False
        in_string = False
        in_char = False
        for match in brace_regex.finditer(lines):
            token = match.group()

            if token == "/*":
                if not in_singlecomment and not in_string and not in_char:
                    in_multicomment = True
            elif token == "*/":
                if not in_singlecomment and not in_string and not in_char:
                    in_multicomment = False
            elif token == "//":
                if not in_multicomment and not in_string and not in_char:
                    in_singlecomment = True
            elif in_singlecomment and linesep in token:
                # Ignore token if it's in a singleline comment. Only check it
                # for newlines to end the comment.
                in_singlecomment = False
            elif in_multicomment or in_singlecomment:
                # Tokens processed after this branch are ignored if they are in
                # comments
                continue
            elif token == '\\"':
                continue
            elif token == '"':
                if not in_char:
                    in_string = not in_string
            elif token == "\\'":
                continue
            elif token == "'":
                if not in_string:
                    in_char = not in_char
            elif in_string or in_char:
                # Tokens processed after this branch are ignored if they are in
                # double or single quotes
                continue
            elif match.group("prefix"):
                brace_count += 1
                name_stack.append((brace_count, match.group("prefix").rstrip()))
            elif "{" in token:
                brace_count += 1
            elif token.startswith("}"):
                output += lines[extract_location : match.start()]
                if (
                    len(name_stack) > 0
                    and name_stack[len(name_stack) - 1][0] == brace_count
                ):
                    output += "}  // " + name_stack.pop()[1]
                else:
                    output += lines[match.start() : match.end()]
                extract_location = match.end()
                brace_count -= 1

        # If input has unprocessed lines, write them to output
        if extract_location < len(lines):
            output += lines[extract_location:]

        return output, True



#!/usr/bin/env python3

import argparse
import math
import multiprocessing as mp
import os
import subprocess
import sys

from wpiformat.bracecomment import BraceComment
from wpiformat.cidentlist import CIdentList
from wpiformat.clangformat import ClangFormat
from wpiformat.clangtidy import ClangTidy
from wpiformat.config import Config
from wpiformat.eofnewline import EofNewline
from wpiformat.gtestname import GTestName
from wpiformat.includeguard import IncludeGuard
from wpiformat.includeorder import IncludeOrder
from wpiformat.javaclass import JavaClass
from wpiformat.jni import Jni
from wpiformat.licenseupdate import LicenseUpdate
from wpiformat.lint import Lint
from wpiformat.pyformat import PyFormat
from wpiformat.stdlib import Stdlib
from wpiformat.task import Task
from wpiformat.usingdeclaration import UsingDeclaration
from wpiformat.usingnamespacestd import UsingNamespaceStd
from wpiformat.whitespace import Whitespace


def filter_ignored_files(names):
    """Returns list of files not in .gitignore.

    Keyword arguments:
    names -- list of files to filter
    """
    # "git check-ignore" misbehaves when the names are separated by "\r\n" on
    # Windows, so os.linesep isn't used here.
    encoded_names = "\n".join(names).encode()

    proc = subprocess.run(
        ["git", "check-ignore", "--no-index", "-n", "-v", "--stdin"],
        input=encoded_names,
        stdout=subprocess.PIPE,
    )
    if proc.returncode == 128:
        raise subprocess.CalledProcessError

    output_list = proc.stdout.decode().split("\n")

    # "git check-ignore" prefixes the names of non-ignored files with "::",
    # wraps names in quotes on Windows, and outputs "\n" line separators on all
    # platforms.
    return [
        name[2:].lstrip().strip('"').replace("\\\\", "\\")
        for name in output_list
        if name[0:2] == "::"
    ]


def proc_init(task_pipeline_copy, verbose1_copy, verbose2_copy):
    """Common initialization for process pool worker.

    Keyword arguments:
    task_pipeline_copy -- task pipeline
    verbose1_copy -- verbose1 flag
    verbose2_copy -- verbose2 flag
    """
    global task_pipeline
    global verbose1
    global verbose2
    global print_lock

    task_pipeline = task_pipeline_copy
    verbose1 = verbose1_copy
    verbose2 = verbose2_copy
    print_lock = mp.Lock()


def proc_pipeline(name):
    """Runs the contents of each file through the task pipeline.

    If the contents were modified at any point, the result is written back out
    to the file.

    Keyword arguments:
    name -- file name string
    """
    config_file = Config(os.path.dirname(name), ".styleguide")
    if verbose1 or verbose2:
        with print_lock:
            print("Processing", name)
            if verbose2:
                print("  with config " + config_file.file_name)
                for subtask in task_pipeline:
                    if subtask.should_process_file(config_file, name):
                        print("  with " + type(subtask).__name__)

    lines = ""
    with open(name, "r", encoding="utf8") as file:
        try:
            lines = file.read()
        except UnicodeDecodeError:
            print(
                f"error: {name} contains characters not in UTF-8. Should this be considered a generated file?"
            )
            return False

    # The success flag is aggregated across multiple file processing results
    all_success = True

    output = lines
    for subtask in task_pipeline:
        if subtask.should_process_file(config_file, name):
            output, success = subtask.run_pipeline(config_file, name, output)
            all_success &= success

    if lines != output:
        with open(name, "wb") as file:
            file.write(output.encode())

    return all_success


def proc_standalone(name):
    """Runs each task on each file.

    Keyword arguments:
    name -- file name string
    """
    config_file = Config(os.path.dirname(name), ".styleguide")
    if verbose2:
        with print_lock:
            print("Processing", name)
            for subtask in task_pipeline:
                if subtask.should_process_file(config_file, name):
                    print("  with " + type(subtask).__name__)

    # The success flag is aggregated across multiple file processing results
    all_success = True

    for subtask in task_pipeline:
        if subtask.should_process_file(config_file, name):
            success = subtask.run_standalone(config_file, name)
            all_success &= success

    return all_success


def chunks(l, max_len):
    """Yield successive chunks from l whose content lengths sum to less than
    max_len.
    """
    out = []
    size = 0
    for i, arg in enumerate(l):
        out.append(arg)
        size += len(arg) + len(" ")
        if i == len(l) - 1 or size + len(l[i + 1]) > max_len:
            yield out
            out = []
            size = 0


def proc_batch(files):
    """Runs each task in the pipeline on batches of files.

    These tasks read and write to the files directly. They are given a list of
    all files at once to avoid spawning too many subprocesses.

    Keyword arguments:
    files -- list of file names

    Returns true if all tasks succeeded.
    """
    all_success = True

    for subtask in task_pipeline:
        work = []
        for name in files:
            config_file = Config(os.path.dirname(name), ".styleguide")
            if subtask.should_process_file(config_file, name):
                work.append(name)

        if work:
            # Conservative estimate for max argument length. 32767 is from the
            # Win32 docs for CreateProcessA(), but the limit appears to be lower
            # than that in practice.
            MAX_WIN32_ARGS_LEN = 32767 * 7 / 8

            for subwork in chunks(work, MAX_WIN32_ARGS_LEN):
                if verbose1 or verbose2:
                    print("Running", type(subtask).__name__)
                    if verbose2:
                        for name in subwork:
                            print("  on", name)
                all_success &= subtask.run_batch(config_file, subwork)

    return all_success


def run_pipeline(task_pipeline, args, files):
    """Spawns process pool for proc_pipeline().

    Keyword arguments:
    task_pipeline -- task pipeline
    args -- command line arguments from argparse
    files -- list of file names to process

    Calls sys.exit(1) if any task fails.
    """
    init_args = (task_pipeline, args.verbose1, args.verbose2)

    with mp.Pool(args.jobs, proc_init, init_args) as pool:
        # Start worker processes for task pipeline
        results = pool.map(proc_pipeline, files)

        if not all(results):
            sys.exit(1)


def run_standalone(task_pipeline, args, files):
    """Spawns process pool for proc_standalone().

    Keyword arguments:
    task_pipeline -- task pipeline
    args -- command line arguments from argparse
    files -- list of file names to process

    Calls sys.exit(1) if any task fails.
    """
    init_args = (task_pipeline, args.verbose1, args.verbose2)

    with mp.Pool(args.jobs, proc_init, init_args) as pool:
        # Start worker processes for task pipeline
        results = pool.map(proc_standalone, files)

        if not all(results):
            sys.exit(1)


def run_batch(task_pipeline, args, file_batches):
    """Spawns process pool for proc_batch().

    Keyword arguments:
    task_pipeline -- task pipeline
    args -- command line arguments from argparse
    file_batches -- list of file names to process

    """
    init_args = (task_pipeline, args.verbose1, args.verbose2)

    with mp.Pool(args.jobs, proc_init, init_args) as pool:
        # Start worker processes for batch tasks
        results = pool.map(proc_batch, file_batches)

        if not all(results):
            sys.exit(1)


def main():
    # Parse command-line arguments
    parser = argparse.ArgumentParser(
        description="Runs all formatting tasks on the code base. This should be invoked from a directory within the project."
    )
    parser.add_argument(
        "-v",
        dest="verbose1",
        action="store_true",
        help="verbosity level 1 (prints names of processed files)",
    )
    parser.add_argument(
        "-vv",
        dest="verbose2",
        action="store_true",
        help="verbosity level 2 (prints names of processed files and tasks run on them)",
    )
    list_files_group = parser.add_mutually_exclusive_group()
    list_files_group.add_argument(
        "-list-all-files",
        dest="list_all_files",
        action="store_true",
        help="list files to be processed instead of processing them",
    )
    list_files_group.add_argument(
        "-list-changed-files",
        dest="list_changed_files",
        action="store_true",
        help="same as list-all-files, but list only files changed from main branch",
    )
    # mp.Pool() uses WaitForMultipleObjects() to wait for subprocess completion
    # on Windows. WaitForMultipleObjects() cannot wait on more then 64 events at
    # once, and mp uses a few internal events. Therefore, the maximum number of
    # parallel jobs is 60.
    cpu_count = min(60, mp.cpu_count())
    parser.add_argument(
        "-j",
        dest="jobs",
        type=int,
        default=cpu_count,
        help="number of jobs to run (default is number of cores)",
    )
    parser.add_argument(
        "-clang",
        dest="clang_version",
        type=str,
        default="",
        help='version suffix for system clang-format (invokes "clang-format-CLANG_VERSION" or PyPi\'s clang-format if no suffix provided)',
    )
    tidy_group = parser.add_mutually_exclusive_group()
    tidy_group.add_argument(
        "-tidy-changed",
        dest="tidy_changed",
        action="store_true",
        help="also runs clang-tidy-CLANG_VERSION on changed files; this requires a compile_commands.json file",
    )
    tidy_group.add_argument(
        "-tidy-all",
        dest="tidy_all",
        action="store_true",
        help="also runs clang-tidy-CLANG_VERSION on all files (this takes a while); this requires a compile_commands.json file",
    )
    parser.add_argument(
        "-compile-commands",
        dest="compile_commands",
        type=str,
        default="",
        help="path to directory containing compile_commands.json; if unset will search in parent paths",
    )
    parser.add_argument(
        "-tidy-extra-args",
        dest="tidy_extra_args",
        type=str,
        default="",
        help='a comma-delimited list of extra arguments for clang-tidy (given "ARG1,ARG2", invokes "clang-tidy -extra-arg -ARG1 -extra-arg -ARG2")',
    )
    parser.add_argument(
        "-f",
        dest="file",
        type=str,
        default="",
        nargs="+",
        help="file or directory names (can be path relative to python invocation directory or absolute path)",
    )
    parser.add_argument(
        "-no-format",
        dest="no_format",
        action="store_true",
        help="disable formatting steps, only run linting",
    )
    args = parser.parse_args()

    # TODO: Remove after deprecation cycle
    if args.clang_version:
        print(
            'warning: "-clang" flag is deprecated for removal in favor of PyPI\'s clang-format and clang-tidy packages'
        )

    # tidy requires compile_commands.json
    if args.tidy_all or args.tidy_changed:
        ccloc = os.path.join(args.compile_commands, "compile_commands.json")
        if not os.path.exists(ccloc):
            print(
                f"error: clang-tidy: {ccloc} not found (try -compile-commands)",
                file=sys.stderr,
            )
            sys.exit(1)

    # All discovered files are relative to Git repo root directory, so find the
    # root.
    root_path = Task.get_repo_root()
    if root_path == "":
        print("Error: not invoked within a Git repository", file=sys.stderr)
        sys.exit(1)

    # If no files explicitly specified
    if not args.file:
        # Delete temporary files from previous incomplete run
        files = [
            os.path.join(dp, f)
            for dp, dn, fn in os.walk(root_path)
            for f in fn
            if f.endswith(".tmp")
        ]
        for f in files:
            os.remove(f)

        # Recursively create list of files in given directory
        files = [os.path.join(dp, f) for dp, dn, fn in os.walk(root_path) for f in fn]

        if not files:
            print("Error: no files found to format", file=sys.stderr)
            sys.exit(1)
    else:
        files = []
        for name in args.file:
            # If a directory was specified, recursively expand it
            if os.path.isdir(name):
                files.extend(
                    [os.path.join(dp, f) for dp, dn, fn in os.walk(name) for f in fn]
                )
            else:
                files.append(name)

    # Throw an error if any files or directories don't exist
    for f in files:
        if not os.path.exists(f):
            print(f"error: {f}: No such file or directory")
            sys.exit(1)

    # Convert relative paths of files to absolute paths
    files = [os.path.abspath(name) for name in files]

    # Don't run tasks on Git metadata
    files = [name for name in files if os.sep + ".git" + os.sep not in name]

    # Don't check for changes in or run tasks on ignored files
    files = filter_ignored_files(files)

    # Determine name of main branch for generated file comparisons
    branch_options = ["master", "main"]
    main_branch = ""
    for branch in branch_options:
        proc = subprocess.run(
            ["git", "rev-parse", "-q", "--verify", branch], stdout=subprocess.DEVNULL
        )
        if proc.returncode == 0:
            main_branch = branch
            break

    if not main_branch:
        print(
            f"error: One of the following branches is required for generated file comparisons, but none exist: {branch_options}."
        )
        sys.exit(1)

    # Create list of all changed files
    output_list = subprocess.check_output(
        ["git", "diff", "--name-only", f"{main_branch}..."], encoding="ascii"
    ).split()
    changed_file_list = [root_path + os.sep + line.strip() for line in output_list]

    # Don't run tasks on modifiable or generated files
    work = []
    for name in files:
        config_file = Config(os.path.dirname(name), ".styleguide")

        if config_file.is_modifiable_file(name):
            continue
        if config_file.is_generated_file(name):
            # Emit warning if a generated file was editted
            if name in changed_file_list:
                print("Warning: generated file '" + name + "' modified")
            continue

        work.append(name)
    files = work

    # If there are no files left, do nothing
    if len(files) == 0:
        sys.exit(0)

    # Handle list-all-files and list-changed-files options
    if args.list_all_files or args.list_changed_files:
        if args.list_changed_files:
            files = list(set(files) & set(changed_file_list))
        for file in files:
            print(file)
        sys.exit(0)

    # Prepare file batches for batch tasks
    chunksize = math.ceil(len(files) / args.jobs)
    file_batches = [files[i : i + chunksize] for i in range(0, len(files), chunksize)]

    if args.no_format:
        # Only run Lint
        task_pipeline = [Lint()]
        run_batch(task_pipeline, args, file_batches)
    else:
        # IncludeOrder is run after Stdlib so any C std headers changed to C++
        # or vice versa are sorted properly. ClangFormat is run after the other
        # tasks so it can clean up their formatting.
        task_pipeline = [
            BraceComment(),
            CIdentList(),
            EofNewline(),
            GTestName(),
            IncludeGuard(),
            LicenseUpdate(),
            JavaClass(),
            Stdlib(),
            IncludeOrder(),
            UsingDeclaration(),
            UsingNamespaceStd(),
            Whitespace(),
            ClangFormat(args.clang_version),
            Jni(),  # Fixes clang-format formatting
        ]
        run_pipeline(task_pipeline, args, files)

        # Lint is run last since previous tasks can affect its output.
        task_pipeline = [PyFormat(), Lint()]
        run_batch(task_pipeline, args, file_batches)

    # ClangTidy is run last of all; it needs the actual files
    if args.tidy_all or args.tidy_changed:
        if args.tidy_changed:
            files = list(set(files) & set(changed_file_list))
        task_pipeline = [
            ClangTidy(
                args.clang_version,
                args.compile_commands,
                args.tidy_extra_args.split(",") if args.tidy_extra_args else [],
            )
        ]
        run_standalone(task_pipeline, args, files)


if __name__ == "__main__":
    main()



"""This task removes trailing whitespace from the file."""

import os

from wpiformat.task import Task


class Whitespace(Task):
    def run_pipeline(self, config_file, name, lines):
        linesep = super().get_linesep(lines)

        return "".join(line.rstrip() + linesep for line in lines.splitlines()), True



"""This task updates the license header at the top of the file."""

from datetime import date
import os
import regex
import subprocess
import sys

from wpiformat.config import Config
from wpiformat.task import Task


class LicenseUpdate(Task):
    @staticmethod
    def should_process_file(config_file, name):
        license_regex = config_file.regex("licenseUpdateExclude")

        return (
            config_file.is_c_file(name)
            or config_file.is_cpp_file(name)
            or name.endswith(".java")
        ) and not license_regex.search(name)

    def __try_regex(self, lines, last_year, license_template):
        """Try finding license with regex of license template.

        Keyword arguments:
        lines -- lines of file
        last_year -- last year in copyright range
        license_template -- license_template string

        Returns:
        Tuple of whether license was found, first year in copyright range, and
        file contents after license.
        """
        linesep = super().get_linesep(lines)

        # Convert the license template to a regex
        license_rgxstr = "^" + linesep.join(license_template)
        license_rgxstr = (
            license_rgxstr.replace("*", r"\*")
            .replace(".", r"\.")
            .replace("(", r"\(")
            .replace(")", r"\)")
            .replace("{year}", r"(?P<year>[0-9]+)(-[0-9]+)?")
            .replace("{padding}", "[ ]*")
        )
        license_rgx = regex.compile(license_rgxstr, regex.M)

        first_year = last_year

        # Compare license
        match = license_rgx.search(lines)
        if match:
            try:
                first_year = match.group("year")
            except IndexError:
                pass

            # If comment at beginning of file is non-empty license, update it
            return True, first_year, linesep + lines[match.end() :].lstrip()
        else:
            return False, first_year, lines

    def __try_string_search(self, lines, last_year, license_template):
        """Try finding license with string search.

        Keyword arguments:
        lines -- lines of file
        last_year -- last year in copyright range
        license_template -- license_template string

        Returns:
        Tuple of whether license was found, first year in copyright range, and
        file contents after license.
        """
        linesep = super().get_linesep(lines)

        # Strip newlines at top of file
        stripped_lines = lines.lstrip().split(linesep)

        # If a comment at the beginning of the file is considered a license, it
        # is replaced with an updated license. Otherwise, a license header is
        # inserted before it.
        first_comment_is_license = False
        license_end = 0

        # Regex for tokenizing on comment boundaries
        token_regex = regex.compile(r"/\*|\*/|^//")

        in_multiline_comment = False
        for line in stripped_lines:
            # If part of comment contains "Copyright (c)", comment is
            # license.
            if "Copyright (c)" in line:
                first_comment_is_license = True

            line_has_comment = False
            for match in token_regex.finditer(line):
                # If any comment token was matched, the line has a comment
                line_has_comment = True

                token = match.group()

                if token == "/*":
                    in_multiline_comment = True
                elif token == "*/":
                    in_multiline_comment = False
            if not in_multiline_comment and not line_has_comment:
                break
            else:
                license_end += 1

        first_year = last_year

        # If comment at beginning of file is non-empty license, update it
        if first_comment_is_license and license_end > 0:
            license_part = linesep.join(stripped_lines[0:license_end])
            appendix_part = (
                linesep + linesep.join(stripped_lines[license_end:]).lstrip()
            )

            year_regex = regex.compile(r"Copyright \(c\)(?>.*?\s(20..))")
            for line in license_part.split(linesep):
                match = year_regex.search(line)
                # If license contains copyright pattern, extract the first year
                if match:
                    first_year = match.group(1)
                    break

            return True, first_year, appendix_part
        else:
            return False, first_year, linesep + lines.lstrip()

    def run_pipeline(self, config_file, name, lines):
        linesep = super().get_linesep(lines)

        _, license_template = Config.read_file(
            os.path.dirname(os.path.abspath(name)), ".styleguide-license"
        )

        # Get year when file was most recently modified in Git history
        #
        # Committer date is used instead of author date (the one shown by "git
        # log") because the year the file was last modified in the history
        # should be used. Author dates can be older than this or even out of
        # order in the log.
        last_year = subprocess.check_output(
            ["git", "log", "-n", "1", "--format=%ci", "--", name]
        ).decode()[:4]

        # Check if file has uncomitted changes in the working directory
        has_uncommitted_changes = subprocess.run(
            ["git", "diff-index", "--quiet", "HEAD", "--", name]
        ).returncode

        # If file hasn't been committed yet or has changes in the working
        # directory, use current calendar year as end of copyright year range
        if last_year == "" or has_uncommitted_changes:
            last_year = str(date.today().year)

        success, first_year, appendix = self.__try_regex(
            lines, last_year, license_template
        )
        if not success:
            success, first_year, appendix = self.__try_string_search(
                lines, last_year, license_template
            )

        output = ""

        # Determine copyright range and trailing padding
        if first_year != last_year:
            year_range = first_year + "-" + last_year
        else:
            year_range = first_year

        for line in license_template:
            # Insert copyright year range
            line = line.replace("{year}", year_range)

            # Insert padding which expands to the 80th column. If there is more
            # than one padding token, the line may contain fewer than 80
            # characters due to rounding during the padding width calculation.
            PADDING_TOKEN = "{padding}"
            padding_count = line.count(PADDING_TOKEN)
            if padding_count:
                padding = 80 - len(line) + len(PADDING_TOKEN) * padding_count
                padding_width = int(padding / padding_count)
                line = line.replace(PADDING_TOKEN, " " * padding_width)

            output += line + linesep

        # Copy rest of original file into new one
        output += appendix

        return output, True



import os

from .test_tasktest import *
from wpiformat.includeorder import IncludeOrder


def test_includeorder():
    test = TaskTest(IncludeOrder())

    # cpp source including related header with wrong include braces and C++ sys
    # before C sys headers
    test.add_input(
        "./Utility.cpp",
        "#include <Utility.h>"
        + os.linesep
        + os.linesep
        + "#include <sstream>"
        + os.linesep
        + os.linesep
        + "#include <cxxabi.h>"
        + os.linesep
        + "#include <execinfo.h>"
        + os.linesep
        + os.linesep
        + '#include "HAL/HAL.h"'
        + os.linesep
        + '#include "Task.h"'
        + os.linesep
        + '#include "nivision.h"'
        + os.linesep,
    )
    test.add_output(
        '#include "Utility.h"'
        + os.linesep
        + os.linesep
        + "#include <cxxabi.h>"
        + os.linesep
        + "#include <execinfo.h>"
        + os.linesep
        + os.linesep
        + "#include <sstream>"
        + os.linesep
        + os.linesep
        + '#include "HAL/HAL.h"'
        + os.linesep
        + '#include "Task.h"'
        + os.linesep
        + '#include "nivision.h"'
        + os.linesep,
        True,
    )

    # Ensure quotes around C and C++ std header includes are replaced with
    # angle brackets and they are properly sorted into two groups
    test.add_input(
        "./Test.h",
        '#include "stdio.h"'
        + os.linesep
        + '#include "iostream"'
        + os.linesep
        + '#include "memory"'
        + os.linesep
        + '#include "signal.h"'
        + os.linesep,
    )
    test.add_output(
        "#include <signal.h>"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + "#include <memory>"
        + os.linesep,
        True,
    )

    # Ensure NOLINT headers have newlines around them
    test.add_input(
        "./Test.h",
        '#include "ctre/PDP.h"  // NOLINT'
        + os.linesep
        + '#include "gtest/gtest.h"  // NOLINT'
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep,
    )
    test.add_output(
        '#include "ctre/PDP.h"  // NOLINT'
        + os.linesep
        + os.linesep
        + '#include "gtest/gtest.h"  // NOLINT'
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep,
        True,
    )

    # Ensure NOLINT headers don't have extra newlines inserted after them
    test.add_input(
        "./Test.h",
        '#include "ctre/PDP.h"  // NOLINT'
        + os.linesep
        + os.linesep
        + '#include "gtest/gtest.h"  // NOLINT'
        + os.linesep
        + os.linesep
        + "namespace wpi {"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Ensure NOLINT headers take precedence over overrides and have newline
    # inserted
    test.add_input(
        "./Test.h",
        '#include "ctre/PDP.h"  // NOLINT'
        + os.linesep
        + "#include <atomic>"
        + os.linesep
        + "#include <condition_variable>"
        + os.linesep
        + "#include <thread>"
        + os.linesep,
    )
    test.add_output(
        '#include "ctre/PDP.h"  // NOLINT'
        + os.linesep
        + os.linesep
        + "#include <atomic>"
        + os.linesep
        + "#include <condition_variable>"
        + os.linesep
        + "#include <thread>"
        + os.linesep,
        True,
    )

    # Check sorting for at least one header from each group except related
    # header. Test.inc isn't considered related in headers.
    test.add_input(
        "./Test.h",
        '#include "MyHeader.h"'
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + '#include "Test.inc"'
        + os.linesep
        + "#include <sys/time.h>"
        + os.linesep
        + "#include <fstream>"
        + os.linesep
        + "#include <boost/algorithm/string/replace.hpp>"
        + os.linesep,
    )
    test.add_output(
        "#include <stdio.h>"
        + os.linesep
        + "#include <sys/time.h>"
        + os.linesep
        + os.linesep
        + "#include <fstream>"
        + os.linesep
        + os.linesep
        + "#include <boost/algorithm/string/replace.hpp>"
        + os.linesep
        + os.linesep
        + '#include "MyHeader.h"'
        + os.linesep
        + '#include "Test.inc"'
        + os.linesep,
        True,
    )

    # Verify "other header" isn't identified as C system include
    test.add_input(
        "./Test.h",
        "#include <OtherHeader.h>" + os.linesep + "#include <sys/time.h>" + os.linesep,
    )
    test.add_output(
        "#include <sys/time.h>"
        + os.linesep
        + os.linesep
        + "#include <OtherHeader.h>"
        + os.linesep,
        True,
    )

    # Verify newline is added between last header and code after it
    test.add_input(
        "./Test.cpp",
        '#include "MyFile.h"'
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + "namespace std {"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "#include <iostream>"
        + os.linesep
        + os.linesep
        + '#include "MyFile.h"'
        + os.linesep
        + os.linesep
        + "namespace std {"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Verify newlines are removed between last header and code after it
    test.add_input(
        "./Test.cpp",
        '#include "MyFile.h"'
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + os.linesep
        + os.linesep
        + os.linesep
        + "namespace std {"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "#include <iostream>"
        + os.linesep
        + os.linesep
        + '#include "MyFile.h"'
        + os.linesep
        + os.linesep
        + "namespace std {"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Ensure extra newlines aren't added between ifdef blocks
    test.add_input(
        "./Test.h",
        "#include <stdint.h>"
        + os.linesep
        + os.linesep
        + "#ifdef __cplusplus"
        + os.linesep
        + "#include <cstddef>"
        + os.linesep
        + "#else"
        + os.linesep
        + "#include <stddef.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#ifdef __cplusplus"
        + os.linesep
        + 'extern "C" {'
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "struct CvMat;"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Ensure headers stay grouped together between license header and other code
    test.add_input(
        "./Test.cpp",
        "// Copyright (c) Company Name 2016."
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + '#include "Test.h"'
        + os.linesep
        + "namespace std {"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "// Copyright (c) Company Name 2016."
        + os.linesep
        + '#include "Test.h"'
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + os.linesep
        + "namespace std {"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Verify headers are sorted across #ifdef
    test.add_input(
        "./Error.h",
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <stdint.h>"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + "#ifdef _WIN32"
        + os.linesep
        + "#include <Windows.h>"
        + os.linesep
        + "// This is a comment"
        + os.linesep
        + "#undef GetMessage"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + '#include "Base.h"'
        + os.linesep
        + '#include "llvm/StringRef.h"'
        + os.linesep,
    )
    test.add_output(
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <stdint.h>"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + "#ifdef _WIN32"
        + os.linesep
        + "#include <Windows.h>"
        + os.linesep
        + "// This is a comment"
        + os.linesep
        + "#undef GetMessage"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + '#include "Base.h"'
        + os.linesep
        + '#include "llvm/StringRef.h"'
        + os.linesep,
        True,
    )

    # Verify "#ifdef _WIN32" acts as barrier for out-of-order includes
    test.add_input(
        "./Error.h",
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <stdint.h>"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + "#ifdef _WIN32"
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + "#include <Windows.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + '#include "Base.h"'
        + os.linesep
        + '#include "llvm/StringRef.h"'
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Verify "#ifdef __linux__" is sorted in correct category below other
    # headers
    test.add_input(
        "./Error.h",
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <stdlib.h>"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + "#ifdef __linux__"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + '#include "Base.h"'
        + os.linesep
        + '#include "llvm/StringRef.h"'
        + os.linesep,
    )
    test.add_output(
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#include <stdlib.h>"
        + os.linesep
        + os.linesep
        + "#ifdef __linux__"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + '#include "Base.h"'
        + os.linesep
        + '#include "llvm/StringRef.h"'
        + os.linesep,
        True,
    )

    # Verify "#ifdef __linux__" is included in output if no headers are in same
    # category
    test.add_input(
        "./Error.h",
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#ifdef __linux__"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#include <string>"
        + os.linesep
        + os.linesep
        + '#include "Base.h"'
        + os.linesep
        + '#include "llvm/StringRef.h"'
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Verify #ifdef blocks in a row are handled properly
    test.add_input(
        "./Log.cpp",
        '#include "Log.h"'
        + os.linesep
        + os.linesep
        + "#include <cstdio>"
        + os.linesep
        + os.linesep
        + "#ifdef _WIN32"
        + os.linesep
        + "#include <cstdlib>"
        + os.linesep
        + "#else"
        + os.linesep
        + "#include <cstring>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#ifdef __APPLE__"
        + os.linesep
        + "#include <libgen.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#ifdef __ANDROID__"
        + os.linesep
        + "#include <libgen.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "using namespace cs;"
        + os.linesep,
    )
    test.add_output(
        '#include "Log.h"'
        + os.linesep
        + os.linesep
        + "#ifdef __APPLE__"
        + os.linesep
        + "#include <libgen.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#ifdef __ANDROID__"
        + os.linesep
        + "#include <libgen.h>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#include <cstdio>"
        + os.linesep
        + os.linesep
        + "#ifdef _WIN32"
        + os.linesep
        + "#include <cstdlib>"
        + os.linesep
        + "#else"
        + os.linesep
        + "#include <cstring>"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "using namespace cs;"
        + os.linesep,
        True,
    )

    # Verify comments aren't mangled
    test.add_input(
        "./cscore.h",
        "/* C API */"
        + os.linesep
        + '#include "cscore_c.h"'
        + os.linesep
        + os.linesep
        + "#ifdef __cplusplus"
        + os.linesep
        + "/* C++ API */"
        + os.linesep
        + '#include "cscore_cpp.h"'
        + os.linesep
        + '#include "cscore_oo.h"'
        + os.linesep
        + "#endif /* __cplusplus */"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Check recursive #ifdef
    test.add_input(
        "./Test.h",
        "#include <algorithm>"
        + os.linesep
        + os.linesep
        + "#ifdef __linux__"
        + os.linesep
        + "#include <sys/socket.h>"
        + os.linesep
        + "#include <sys/syscall.h>"
        + os.linesep
        + "#ifdef __GNUC__ > 4"
        + os.linesep
        + "#include <algorithm>"
        + os.linesep
        + "#endif"
        + os.linesep
        + "#endif"
        + os.linesep,
    )
    test.add_output(
        "#include <algorithm>"
        + os.linesep
        + os.linesep
        + "#ifdef __linux__"
        + os.linesep
        + "#include <sys/socket.h>"
        + os.linesep
        + "#include <sys/syscall.h>"
        + os.linesep
        + os.linesep
        + "#ifdef __GNUC__ > 4"
        + os.linesep
        + "#include <algorithm>"
        + os.linesep
        + "#endif"
        + os.linesep
        + "#endif"
        + os.linesep,
        True,
    )

    # Verify extra newline from #endif is removed
    test.add_input(
        "./Test.h",
        '#include "HAL/HAL.h"'
        + os.linesep
        + '#include "NotifyListener.h"'
        + os.linesep
        + os.linesep
        + "#ifdef __cplusplus"
        + os.linesep
        + 'extern "C" {'
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "void HALSIM_ResetSPIAccelerometerData(int32_t index);"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Large test
    test.add_input(
        "./UsbCameraImpl.cpp",
        "#include <algorithm>"
        + os.linesep
        + os.linesep
        + '#include "Handle.h"'
        + os.linesep
        + '#include "Log.h"'
        + os.linesep
        + '#include "Notifier.h"'
        + os.linesep
        + '#include "UsbUtil.h"'
        + os.linesep
        + '#include "c_util.h"'
        + os.linesep
        + '#include "cscore_cpp.h"'
        + os.linesep
        + os.linesep
        + "#ifdef __linux__"
        + os.linesep
        + "#include <dirent.h>"
        + os.linesep
        + "#include <fcntl.h>"
        + os.linesep
        + "#include <libgen.h>"
        + os.linesep
        + "#include <linux/kernel.h>"
        + os.linesep
        + "#include <linux/types.h>"
        + os.linesep
        + "#include <linux/videodev2.h>"
        + os.linesep
        + "#include <sys/eventfd.h>"
        + os.linesep
        + "#include <sys/inotify.h>"
        + os.linesep
        + "#include <sys/ioctl.h>"
        + os.linesep
        + "#include <sys/select.h>"
        + os.linesep
        + "#include <sys/stat.h>"
        + os.linesep
        + "#include <sys/time.h>"
        + os.linesep
        + "#include <sys/types.h>"
        + os.linesep
        + "#include <unistd.h>"
        + os.linesep
        + os.linesep
        + "#elif defined(_WIN32)"
        + os.linesep
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "using namespace cs;"
        + os.linesep,
    )
    test.add_output(
        "#ifdef __linux__"
        + os.linesep
        + "#include <dirent.h>"
        + os.linesep
        + "#include <fcntl.h>"
        + os.linesep
        + "#include <libgen.h>"
        + os.linesep
        + "#include <linux/kernel.h>"
        + os.linesep
        + "#include <linux/types.h>"
        + os.linesep
        + "#include <linux/videodev2.h>"
        + os.linesep
        + "#include <sys/eventfd.h>"
        + os.linesep
        + "#include <sys/inotify.h>"
        + os.linesep
        + "#include <sys/ioctl.h>"
        + os.linesep
        + "#include <sys/select.h>"
        + os.linesep
        + "#include <sys/stat.h>"
        + os.linesep
        + "#include <sys/time.h>"
        + os.linesep
        + "#include <sys/types.h>"
        + os.linesep
        + "#include <unistd.h>"
        + os.linesep
        + "#elif defined(_WIN32)"
        + os.linesep
        + "#endif"
        + os.linesep
        + os.linesep
        + "#include <algorithm>"
        + os.linesep
        + os.linesep
        + '#include "Handle.h"'
        + os.linesep
        + '#include "Log.h"'
        + os.linesep
        + '#include "Notifier.h"'
        + os.linesep
        + '#include "UsbUtil.h"'
        + os.linesep
        + '#include "c_util.h"'
        + os.linesep
        + '#include "cscore_cpp.h"'
        + os.linesep
        + os.linesep
        + "using namespace cs;"
        + os.linesep,
        True,
    )

    # Verify relevant headers are found and sorted correctly
    test.add_input(
        "./PDP.cpp",
        "#include <memory>"
        + os.linesep
        + os.linesep
        + '#include "HAL/PDP.h"'
        + os.linesep
        + os.linesep
        + '#include "ctre/PDP.h"'
        + os.linesep
        + os.linesep
        + "using namespace hal;"
        + os.linesep,
    )
    test.add_output(
        '#include "HAL/PDP.h"'
        + os.linesep
        + os.linesep
        + "#include <memory>"
        + os.linesep
        + os.linesep
        + '#include "ctre/PDP.h"'
        + os.linesep
        + os.linesep
        + "using namespace hal;"
        + os.linesep,
        True,
    )

    # Check for idempotence
    test.add_input(
        "./PDP.cpp",
        '#include "HAL/PDP.h"'
        + os.linesep
        + os.linesep
        + "#include <memory>"
        + os.linesep
        + os.linesep
        + '#include "ctre/PDP.h"'
        + os.linesep
        + os.linesep
        + "using namespace hal;"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Verify subgroups are sorted
    test.add_input(
        "./PDP.cpp",
        '#include "support/jni_util.h"'
        + os.linesep
        + '#include "llvm/SmallString.h"'
        + os.linesep
        + '#include "llvm/raw_ostream.h"'
        + os.linesep,
    )
    test.add_output(
        '#include "llvm/SmallString.h"'
        + os.linesep
        + '#include "llvm/raw_ostream.h"'
        + os.linesep
        + '#include "support/jni_util.h"'
        + os.linesep,
        True,
    )

    # Verify duplicate headers are removed
    test.add_input(
        "./PDP.cpp",
        "#include <memory>"
        + os.linesep
        + '#include "ctre/PDP.h"'
        + os.linesep
        + os.linesep
        + '#include "HAL/PDP.h"'
        + os.linesep
        + os.linesep
        + '#include "ctre/PDP.h"'
        + os.linesep
        + "#include <memory>"
        + os.linesep
        + os.linesep
        + "using namespace hal;"
        + os.linesep,
    )
    test.add_output(
        '#include "HAL/PDP.h"'
        + os.linesep
        + os.linesep
        + "#include <memory>"
        + os.linesep
        + os.linesep
        + '#include "ctre/PDP.h"'
        + os.linesep
        + os.linesep
        + "using namespace hal;"
        + os.linesep,
        True,
    )

    # Verify source file inclusion is disallowed
    test.add_input(
        "./Test.h",
        "#include <memory>"
        + os.linesep
        + os.linesep
        + '#include "Stuff.cpp"'
        + os.linesep,
    )
    test.add_latest_input_as_output(False)

    # Ensure commented-out includes are still sorted
    test.add_input(
        "./Test.h",
        '#include "stdio.h"'
        + os.linesep
        + '#include "iostream"'
        + os.linesep
        + '// #include "memory"'
        + os.linesep
        + '#include "signal.h"'
        + os.linesep,
    )
    test.add_output(
        "#include <signal.h>"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + "// #include <memory>"
        + os.linesep,
        True,
    )

    # Ensure commented-out includes with space before them are still sorted
    test.add_input(
        "./Test.h",
        " //#include <signal.h>"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + "#include <memory>"
        + os.linesep,
    )
    test.add_output(
        "//#include <signal.h>"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + "#include <memory>"
        + os.linesep,
        True,
    )

    # Ensure includes with no space between #include and bracket are sorted
    test.add_input(
        "./Test.h",
        "#include<signal.h>"
        + os.linesep
        + "#include<stdio.h>"
        + os.linesep
        + os.linesep
        + "#include<iostream>"
        + os.linesep
        + "#include<memory>"
        + os.linesep,
    )
    test.add_output(
        "#include <signal.h>"
        + os.linesep
        + "#include <stdio.h>"
        + os.linesep
        + os.linesep
        + "#include <iostream>"
        + os.linesep
        + "#include <memory>"
        + os.linesep,
        True,
    )

    # Ensure lines containing #include that aren't includes are not processed
    test.add_input("./Test.h", "// #included here" + os.linesep)
    test.add_latest_input_as_output(True)

    # Ensure extra newline isn't inserted between #pragma and #ifdef
    test.add_input(
        "./Test.h",
        "#pragma once"
        + os.linesep
        + os.linesep
        + "#ifdef _MSC_VER"
        + os.linesep
        + "#endif"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    test.run(OutputType.FILE)



"""Provides a task base class for use by format.py.

format.py excludes matches for the "modifiable" regex before checking for
modifications to generated files because some of the regexes from each group
overlap.
"""

from abc import *
import os


class Task:
    __metaclass__ = ABCMeta

    @staticmethod
    def get_linesep(lines):
        """Returns string containing autodetected line separator for file.

        Keyword arguments:
        lines -- file contents string
        """
        # Find potential line separator
        pos = lines.find("\n")

        # If a newline character was found and the character preceding it is a
        # carriage return, assume CRLF line endings. LF line endings are assumed
        # for empty files.
        if pos > 0 and lines[pos - 1] == "\r":
            return "\r\n"
        else:
            return "\n"

    @staticmethod
    def get_repo_root():
        """Returns the Git repository root as an absolute path.

        An empty string is returned if no repository root was found.
        """
        current_dir = os.path.abspath(os.getcwd())
        while current_dir != os.path.dirname(current_dir):
            if os.path.exists(current_dir + os.sep + ".git"):
                return current_dir
            current_dir = os.path.dirname(current_dir)
        return ""

    @staticmethod
    @abstractmethod
    def should_process_file(config_file, name):
        """Returns true if file should be processed by this task.

        Keyword arguments:
        config_file -- Config object
        name -- file name string

        Process any file by default.
        """
        return True

    @abstractmethod
    def run_pipeline(self, config_file, name, lines):
        """Performs task on file with given lines.

        This function is for processing the file in a pipeline of tasks.

        Keyword arguments:
        config_file -- Config object
        name -- file name string
        lines -- file contents string

        Returns tuple containing processed lines and whether task succeeded in
        processing the file.
        """
        return ("", True)

    @staticmethod
    @abstractmethod
    def run_batch(config_file, names):
        """Performs task on list of files.

        This function is for processing multiple files in one task to reduce
        overhead.

        Keyword arguments:
        config_file -- Config object
        names -- list of file name strings

        Returns True if task succeeded in formatting the files.
        """
        return True



"""This task runs clang-tidy on the file."""

import subprocess
import sys

import clang_tidy

from wpiformat.task import Task


class ClangTidy(Task):
    def __init__(self, clang_version, compile_commands, extra_args):
        """Constructor for ClangTidy task.

        Keyword arguments:
        clang_version -- version number of clang-tidy appended to executable
                         name (deprecated for removal)
        compile_commands -- directory containing compile_commands.json
        extra_args -- list of extra arguments to clang-tidy
        """
        super().__init__()

        if clang_version == "":
            self.exec_name = clang_tidy._get_executable("clang-tidy")
        else:
            self.exec_name = "clang-tidy-" + clang_version

        self.args = ["--quiet"]
        if compile_commands:
            self.args += ["-p", compile_commands]

        # Prepend a dash to the argument here because the main argument parser
        # treats strings with a dash prefix as a new argument instead of the
        # value for -extra-args
        for arg in extra_args:
            self.args += ["-extra-arg", "-" + arg]

    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_cpp_file(name)

    def run_standalone(self, config_file, name):
        try:
            stdout = subprocess.run(
                [self.exec_name] + self.args + [name],
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                encoding="utf-8",
                check=False,
            ).stdout
        except FileNotFoundError:
            print(
                f"error: {self.exec_name} not found in PATH. Is it installed?",
                file=sys.stderr,
            )
            return False

        lines = [l for l in stdout.rstrip().split("\n") if l]

        # Filter out "X error(s) and Y warning(s) generated." lines
        lines = [l for l in lines if " generated." not in l]

        # Filter out "Error while processing" lines
        lines = [l for l in lines if "Error while processing" not in l]

        # Ignore include file not found errors
        filtered_lines = []
        iterlines = iter(lines)
        for l in iterlines:
            if "file not found [clang-diagnostic-error]" in l:
                # Skip #include line and caret indicator line
                next(iterlines)
                next(iterlines)
            else:
                filtered_lines.append(l)
        lines = filtered_lines

        # If any lines are non-empty, print them and report an error
        if any(len(l.rstrip()) > 0 for l in lines):
            print(f"== clang-tidy {name} ==\n" + "\n".join(lines))
            return False

        return True



"""This class is for writing unit tests for tasks."""

from enum import Enum
import io
import os
import sys

from wpiformat.config import Config


class OutputType(Enum):
    FILE = 1
    STDOUT = 2


class TaskTest:
    def __init__(self, task):
        """Constructor for Test object.

        Keyword arguments:
        task -- task on which to run tests
        """
        self.task = task
        self.inputs = []
        self.outputs = []

    def reset(self):
        """Clears input and output lists."""
        self.inputs = []
        self.outputs = []

    def add_input(self, file_name, contents):
        """Adds the given file and its contents to the input list.

        Keyword arguments:
        file_name -- file name string
        contents -- file contents string
        """
        self.inputs.append((file_name, contents))

    def add_output(self, contents, succeeded):
        """Adds the given file contents and whether processing of the input list
        was successful.

        Keyword arguments:
        contents -- file contents string
        succeeded -- whether processing of the input list was successful
        """
        self.outputs.append((contents, succeeded))

    def add_latest_input_as_output(self, succeeded):
        """Adds the latest input to the output list.

        Keyword arguments:
        succeeded -- whether processing of the input list was successful
        """
        self.add_output(self.inputs[len(self.inputs) - 1][1], succeeded)

    def run(self, output_type):
        """Runs the task on each input list element, then compares the resulting
        output against the corresponding output list element.

        output_type == FILE: output list contains file contents
        output_type == STDOUT: output list contains stdout contents

        Keyword Arguments:
        output_type -- the type of output stored in the output list
        """
        assert len(self.inputs) == len(self.outputs)

        config_file = Config(os.path.abspath(os.getcwd()), ".styleguide")

        for i in range(len(self.inputs)):
            if self.task.should_process_file(config_file, self.inputs[i][0]):
                print("Running test {}...".format(i))

                if output_type == OutputType.FILE:
                    output, success = self.task.run_pipeline(
                        config_file, self.inputs[i][0], self.inputs[i][1]
                    )
                elif output_type == OutputType.STDOUT:
                    saved_stdout = sys.stdout
                    sys.stdout = io.StringIO()
                    _output, success = self.task.run_pipeline(
                        config_file, self.inputs[i][0], self.inputs[i][1]
                    )
                    sys.stdout.seek(0)
                    output = sys.stdout.read()
                    sys.stdout = saved_stdout
            else:
                output = self.inputs[i][1]
                success = True

            assert output == self.outputs[i][0]
            assert success == self.outputs[i][1]



"""This task formats JNI signatures according to javah's output.

The format is of the form:

JNIEXPORT <return type> JNICALL
Java_blah
  (args, more args)
{
  // code here

If string of arguments exceeds 80 characters in length, it wraps as follows:

  (args,
   more args)
{
  // code here

The preceding comment containing the Class, Method, and Signature is also
automatically generated based on the function's return type and arguments.
"""

import regex

from wpiformat.task import Task


class Jni(Task):
    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_cpp_src_file(name)

    @staticmethod
    def map_jni_type(type_name):
        ret = ""
        if type_name.endswith("*") or type_name.endswith("Array"):
            ret += "["

        if type_name.startswith("jboolean"):
            return ret + "Z"
        elif type_name.startswith("jbyte"):
            return ret + "B"
        elif type_name.startswith("jchar"):
            return ret + "C"
        elif type_name.startswith("jshort"):
            return ret + "S"
        elif type_name.startswith("jint"):
            return ret + "I"
        elif type_name.startswith("jlong"):
            return ret + "J"
        elif type_name.startswith("jfloat"):
            return ret + "F"
        elif type_name.startswith("jdouble"):
            return ret + "D"
        elif type_name.startswith("void"):
            return ret + "V"
        elif type_name.startswith("jstring"):
            return ret + "Ljava/lang/String;"
        elif type_name.startswith("jobject"):
            return ret + "Ljava/lang/Object;"
        else:
            return ret + "?"

    def run_pipeline(self, config_file, name, lines):
        linesep = super().get_linesep(lines)

        regex_str_sig = (
            r"(/\*(?>(.|\n)*?\*/)\s+)?"
            + r"JNIEXPORT\s+(?P<ret>\w+)\s+JNICALL\s+"
            + r"(?P<func>Java_\w+)\s*\(\s*"
            + r"(?P<env_type>JNIEnv\s*\*\s*)"
            + r"(?P<env_name>\w+)?,\s*"
            + r"(?P<param_type>jclass|jobject)\s*(?P<param_name>\w*)?"
        )
        regex_sig = regex.compile(regex_str_sig)

        regex_str_func = r"Java_(?P<class>\w+)_(?P<method>[^_]+)$"
        regex_func = regex.compile(regex_str_func)

        # Matches a comma followed by the type, an optional variable name, and
        # an optional closing parenthesis
        regex_str_arg = (
            r", \s* (?P<arg>(?P<arg_type>[\w\*]+)(\s+ \w+)?)|\)\s*" r"(?P<trailing>{|;)"
        )
        regex_arg = regex.compile(regex_str_arg, regex.VERBOSE)

        output = ""
        pos = 0
        for match_sig in regex_sig.finditer(lines):
            comment = ""
            signature = ""

            if match_sig.start() > 0:
                output += lines[pos : match_sig.start()]

            # Add JNI-specific args
            jni_args = "  ("
            if match_sig.group("env_type"):
                jni_args += match_sig.group("env_type")
            if match_sig.group("env_name"):
                jni_args += match_sig.group("env_name")
            jni_args += ", " + match_sig.group("param_type")
            if match_sig.group("param_name"):
                jni_args += " " + match_sig.group("param_name")

            # Write JNI function comment. Splitting at "__" removes overload
            # annotation from method comment
            match = regex_func.search(match_sig.group("func").split("__")[0])
            comment += (
                "/*"
                + linesep
                + " * Class:     "
                + match.group("class")
                + linesep
                + " * Method:    "
                + match.group("method")
                + linesep
                + " * Signature: ("
            )

            signature += (
                "JNIEXPORT "
                + match_sig.group("ret")
                + " JNICALL"
                + linesep
                + match_sig.group("func")
                + linesep
                + jni_args
            )

            # Add other args
            line_length = len(jni_args)
            for match_arg in regex_arg.finditer(lines[match_sig.end() :]):
                if ")" in match_arg.group():
                    break
                # If args going past 80 characters
                elif (
                    line_length + len(", ") + len(match_arg.group("arg")) + len(")")
                    > 80
                ):
                    # Put current arg on next line and set line_length to
                    # reflect that
                    signature += "," + linesep + "   " + match_arg.group("arg")
                    line_length = len("   " + match_arg.group("arg"))
                else:
                    signature += ", " + match_arg.group("arg")
                    line_length += len(", ") + len(match_arg.group("arg"))
                comment += self.map_jni_type(match_arg.group("arg_type"))
            comment += (
                ")"
                + self.map_jni_type(match_sig.group("ret"))
                + linesep
                + " */"
                + linesep
            )

            # Output correct trailing character for declaration vs definition
            if match_arg.group("trailing") == "{":
                signature += ")" + linesep + "{"
            else:
                signature += ");"

            output += comment + signature

            pos = match_sig.end() + match_arg.end()

        # Write rest of file
        if pos < len(lines):
            output += lines[pos:]

        if output == "":
            return lines, True
        else:
            return output, True



"""This task runs black on files with Python extension."""

import subprocess
import sys

from wpiformat.task import Task


class PyFormat(Task):
    @staticmethod
    def should_process_file(config_file, name):
        return name.endswith(".py")

    @staticmethod
    def run_batch(config_file, names):
        try:
            args = [sys.executable, "-m", "black", "-q"]
            returncode = subprocess.run(args + names).returncode
        except FileNotFoundError:
            print("Error: black not found in PATH. Is it installed?", file=sys.stderr)
            return False
        return True



import os

from .test_tasktest import *
from wpiformat.includeguard import IncludeGuard
from wpiformat.task import Task


def test_includeguard():
    test = TaskTest(IncludeGuard())

    repo_root = os.path.basename(Task.get_repo_root()).upper()

    # Fix incorrect include guard
    test.add_input(
        "./Test.h",
        "#ifndef WRONG_H"
        + os.linesep
        + "#define WRONG_C"
        + os.linesep
        + os.linesep
        + "#endif"
        + os.linesep,
    )
    test.add_output(
        "#ifndef "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_TEST_H_"
        + os.linesep,
        True,
    )

    # Ensure nested preprocessor statements are handled properly for incorrect
    # include guard
    test.add_input(
        "./Test.h",
        "#ifndef WRONG_H"
        + os.linesep
        + "#define WRONG_C"
        + os.linesep
        + os.linesep
        + "#if SOMETHING"
        + os.linesep
        + "// do something"
        + os.linesep
        + "#endif"
        + os.linesep
        + "#endif"
        + os.linesep,
    )
    test.add_output(
        "#ifndef "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + os.linesep
        + "#if SOMETHING"
        + os.linesep
        + "// do something"
        + os.linesep
        + "#endif"
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_TEST_H_"
        + os.linesep,
        True,
    )

    # Don't touch correct include guard
    test.add_input(
        "./Test.h",
        "#ifndef "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_TEST_H_"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Fail on missing include guard
    test.add_input("./Test.h", "// Empty file" + os.linesep)
    test.add_latest_input_as_output(False)

    # Verify pragma once counts as include guard
    test.add_input("./Test.h", "#pragma once" + os.linesep)
    test.add_latest_input_as_output(True)

    # Ensure include guard roots are processed correctly
    test.add_input(
        "./Test.h",
        "#ifndef "
        + repo_root
        + "_WPIFORMAT_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_WPIFORMAT_TEST_H_"
        + os.linesep
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_WPIFORMAT_TEST_H_"
        + os.linesep,
    )
    test.add_output(
        "#ifndef "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_TEST_H_"
        + os.linesep,
        True,
    )

    # Ensure leading underscores are removed (this occurs if the user doesn't
    # include a trailing "/" in the include guard root)
    test.add_input(
        "./Test/Test.h",
        "#ifndef "
        + repo_root
        + "_WPIFORMAT_TEST_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_WPIFORMAT_TEST_TEST_H_"
        + os.linesep
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_WPIFORMAT_TEST_TEST_H_"
        + os.linesep,
    )
    test.add_output(
        "#ifndef "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + "#define "
        + repo_root
        + "_TEST_H_"
        + os.linesep
        + os.linesep
        + "#endif  // "
        + repo_root
        + "_TEST_H_"
        + os.linesep,
        True,
    )

    test.run(OutputType.FILE)



import io
import os
import sys

from .test_tasktest import *
from wpiformat.usingdeclaration import UsingDeclaration


def test_usingdeclaration():
    test = TaskTest(UsingDeclaration())

    # Before class block
    test.add_input(
        "./Test.h",
        "using std::chrono;"
        + os.linesep
        + "class Test {"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("./Test.h: 1: 'using std::chrono;' in global namespace\n", False)

    # Inside enum block
    test.add_input(
        "./Test.h",
        "enum Test {"
        + os.linesep
        + "  using std::chrono;"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("", True)

    # After { block
    test.add_input(
        "./Test.h",
        "{" + os.linesep + "}" + os.linesep + "using std::chrono;" + os.linesep,
    )
    test.add_output("./Test.h: 3: 'using std::chrono;' in global namespace\n", False)

    # Before class block with NOLINT
    test.add_input(
        "./Test.h",
        "using std::chrono;  // NOLINT"
        + os.linesep
        + "class Test {"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("", True)

    # "using" in comment without trailing semicolon
    test.add_input(
        "./Test.h",
        "// using"
        + os.linesep
        + "void func() {"
        + os.linesep
        + "  using A = int;"
        + os.linesep
        + "  using B = int;"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output("", True)

    test.run(OutputType.STDOUT)



import os

from .test_tasktest import *
from wpiformat.jni import Jni


def test_jni():
    test = TaskTest(Jni())

    # Input args go to next line even if they fit on same line
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_TestJNI_testFunc(JNIEnv* env, jclass) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     TestJNI"
        + os.linesep
        + " * Method:    testFunc"
        + os.linesep
        + " * Signature: ()V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_TestJNI_testFunc"
        + os.linesep
        + "  (JNIEnv* env, jclass)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Input aligned to "(" and args past end of line
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_setCameraExposureHoldCurrent(JNIEnv* env,"
        + os.linesep
        + "                                                                 jclass,"
        + os.linesep
        + "                                                                 jint source) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_cscore_CameraServerJNI"
        + os.linesep
        + " * Method:    setCameraExposureHoldCurrent"
        + os.linesep
        + " * Signature: (I)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_setCameraExposureHoldCurrent"
        + os.linesep
        + "  (JNIEnv* env, jclass, jint source)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Args in input on line after "(" and args length > 80 characters
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT void JNICALL Java_edu_wpi_cscore_CameraServerJNI_putSourceFrame("
        + os.linesep
        + "    JNIEnv *env, jclass, jint source, jlong imageNativeObj) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_cscore_CameraServerJNI"
        + os.linesep
        + " * Method:    putSourceFrame"
        + os.linesep
        + " * Signature: (IJ)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_putSourceFrame"
        + os.linesep
        + "  (JNIEnv *env, jclass, jint source, jlong imageNativeObj)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Args > 80 characters long
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT jint JNICALL Java_edu_wpi_cscore_CameraServerJNI_createSourceProperty("
        + os.linesep
        + "    JNIEnv *env, jclass, jint source, jstring name, jint kind, jint minimum,"
        + os.linesep
        + "    jint maximum, jint step, jint defaultValue, jint value) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_cscore_CameraServerJNI"
        + os.linesep
        + " * Method:    createSourceProperty"
        + os.linesep
        + " * Signature: (ILjava/lang/String;IIIIII)I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_createSourceProperty"
        + os.linesep
        + "  (JNIEnv *env, jclass, jint source, jstring name, jint kind, jint minimum,"
        + os.linesep
        + "   jint maximum, jint step, jint defaultValue, jint value)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Ensure fixes clang-format output aligned with "("
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance(JNIEnv*,"
        + os.linesep
        + "                                                                 jclass) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv*, jclass)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Idempotence for same code
    test.add_input(
        "./TestJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv*, jclass)"
        + os.linesep
        + "{"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Idempotence for same code with named jclass variable
    test.add_input(
        "./TestJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv*, jclass class)"
        + os.linesep
        + "{"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Check signature that breaks verbose regexes
    test.add_input(
        "./NetworkTablesJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntry"
        + os.linesep
        + " * Signature: (ILjava/lang/String;)I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntry(JNIEnv* env, jclass,"
        + os.linesep
        + "                                                           jint inst,"
        + os.linesep
        + "                                                           jstring key) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntry"
        + os.linesep
        + " * Signature: (ILjava/lang/String;)I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntry"
        + os.linesep
        + "  (JNIEnv* env, jclass, jint inst, jstring key)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Function with array type as argument
    test.add_input(
        "./NetworkTablesJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntries"
        + os.linesep
        + " * Signature: (ILjava/lang/String;I)[I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jintArray JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntries(JNIEnv* env,"
        + os.linesep
        + "                                                             jclass, jint inst,"
        + os.linesep
        + "                                                             jstring prefix,"
        + os.linesep
        + "                                                             jint types) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntries"
        + os.linesep
        + " * Signature: (ILjava/lang/String;I)[I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jintArray JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntries"
        + os.linesep
        + "  (JNIEnv* env, jclass, jint inst, jstring prefix, jint types)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Ensure functions with overloads are handled correctly
    test.add_input(
        "./NetworkTablesJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    setRaw"
        + os.linesep
        + " * Signature: (IJ[BZ)Z"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jboolean JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_setRaw__IJ_3BZ"
        + os.linesep
        + "  (JNIEnv* env, jclass, jint entry, jlong time, jbyteArray value,"
        + os.linesep
        + "   jboolean force)"
        + os.linesep
        + "{"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Ensure text before JNIEXPORT and after args and ")" is handled correctly
    # as well as two JNI functions in a row
    test.add_input(
        "./TestJNI.cpp",
        "/**"
        + os.linesep
        + " *"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getDefaultInstance"
        + os.linesep
        + "  (JNIEnv *, jclass)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::GetDefaultInstance();"
        + os.linesep
        + "}"
        + os.linesep
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv *, jclass)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::CreateInstance();"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getDefaultInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getDefaultInstance"
        + os.linesep
        + "  (JNIEnv *, jclass)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::GetDefaultInstance();"
        + os.linesep
        + "}"
        + os.linesep
        + os.linesep
        + "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv *, jclass)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::CreateInstance();"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Handle function declarations properly
    test.add_input(
        "./TestJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getDefaultInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getDefaultInstance"
        + os.linesep
        + "  (JNIEnv *, jclass);"
        + os.linesep
        + os.linesep
        + "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv *, jclass)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::CreateInstance();"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Handle functions whose arguments don't have variable names properly
    test.add_input(
        "./DigitalGlitchFilterJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI"
        + os.linesep
        + " * Method:    cleanFilter"
        + os.linesep
        + " * Signature: (I)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL Java_edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI_cleanFilter"
        + os.linesep
        + "  (JNIEnv *, jclass, jint)"
        + os.linesep
        + "{"
        + os.linesep
        + "  HAL_CleanFilter(handle);"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI"
        + os.linesep
        + " * Method:    cleanFilter"
        + os.linesep
        + " * Signature: (I)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI_cleanFilter"
        + os.linesep
        + "  (JNIEnv *, jclass, jint)"
        + os.linesep
        + "{"
        + os.linesep
        + "  HAL_CleanFilter(handle);"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Input args go to next line even if they fit on same line
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_TestJNI_testFunc(JNIEnv* env, jobject) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     TestJNI"
        + os.linesep
        + " * Method:    testFunc"
        + os.linesep
        + " * Signature: ()V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_TestJNI_testFunc"
        + os.linesep
        + "  (JNIEnv* env, jobject)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Input aligned to "(" and args past end of line
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_setCameraExposureHoldCurrent(JNIEnv* env,"
        + os.linesep
        + "                                                                 jobject,"
        + os.linesep
        + "                                                                 jint source) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_cscore_CameraServerJNI"
        + os.linesep
        + " * Method:    setCameraExposureHoldCurrent"
        + os.linesep
        + " * Signature: (I)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_setCameraExposureHoldCurrent"
        + os.linesep
        + "  (JNIEnv* env, jobject, jint source)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Args in input on line after "(" and args length > 80 characters
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT void JNICALL Java_edu_wpi_cscore_CameraServerJNI_putSourceFrame("
        + os.linesep
        + "    JNIEnv *env, jobject, jint source, jlong imageNativeObj) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_cscore_CameraServerJNI"
        + os.linesep
        + " * Method:    putSourceFrame"
        + os.linesep
        + " * Signature: (IJ)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_putSourceFrame"
        + os.linesep
        + "  (JNIEnv *env, jobject, jint source, jlong imageNativeObj)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Args > 80 characters long
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT jint JNICALL Java_edu_wpi_cscore_CameraServerJNI_createSourceProperty("
        + os.linesep
        + "    JNIEnv *env, jobject, jint source, jstring name, jint kind, jint minimum,"
        + os.linesep
        + "    jint maximum, jint step, jint defaultValue, jint value) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_cscore_CameraServerJNI"
        + os.linesep
        + " * Method:    createSourceProperty"
        + os.linesep
        + " * Signature: (ILjava/lang/String;IIIIII)I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_cscore_CameraServerJNI_createSourceProperty"
        + os.linesep
        + "  (JNIEnv *env, jobject, jint source, jstring name, jint kind, jint minimum,"
        + os.linesep
        + "   jint maximum, jint step, jint defaultValue, jint value)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Ensure fixes clang-format output aligned with "("
    test.add_input(
        "./TestJNI.cpp",
        "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance(JNIEnv*,"
        + os.linesep
        + "                                                                 jobject) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv*, jobject)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Idempotence for same code
    test.add_input(
        "./TestJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv*, jobject)"
        + os.linesep
        + "{"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Idempotence for same code with named jobject variable
    test.add_input(
        "./TestJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv*, jobject class)"
        + os.linesep
        + "{"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Check signature that breaks verbose regexes
    test.add_input(
        "./NetworkTablesJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntry"
        + os.linesep
        + " * Signature: (ILjava/lang/String;)I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntry(JNIEnv* env, jobject,"
        + os.linesep
        + "                                                           jint inst,"
        + os.linesep
        + "                                                           jstring key) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntry"
        + os.linesep
        + " * Signature: (ILjava/lang/String;)I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntry"
        + os.linesep
        + "  (JNIEnv* env, jobject, jint inst, jstring key)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Function with array type as argument
    test.add_input(
        "./NetworkTablesJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntries"
        + os.linesep
        + " * Signature: (ILjava/lang/String;I)[I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jintArray JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntries(JNIEnv* env,"
        + os.linesep
        + "                                                             jobject, jint inst,"
        + os.linesep
        + "                                                             jstring prefix,"
        + os.linesep
        + "                                                             jint types) {"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getEntries"
        + os.linesep
        + " * Signature: (ILjava/lang/String;I)[I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jintArray JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getEntries"
        + os.linesep
        + "  (JNIEnv* env, jobject, jint inst, jstring prefix, jint types)"
        + os.linesep
        + "{"
        + os.linesep,
        True,
    )

    # Ensure functions with overloads are handled correctly
    test.add_input(
        "./NetworkTablesJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    setRaw"
        + os.linesep
        + " * Signature: (IJ[BZ)Z"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jboolean JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_setRaw__IJ_3BZ"
        + os.linesep
        + "  (JNIEnv* env, jobject, jint entry, jlong time, jbyteArray value,"
        + os.linesep
        + "   jboolean force)"
        + os.linesep
        + "{"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Ensure text before JNIEXPORT and after args and ")" is handled correctly
    # as well as two JNI functions in a row
    test.add_input(
        "./TestJNI.cpp",
        "/**"
        + os.linesep
        + " *"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getDefaultInstance"
        + os.linesep
        + "  (JNIEnv *, jobject)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::GetDefaultInstance();"
        + os.linesep
        + "}"
        + os.linesep
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv *, jobject)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::CreateInstance();"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getDefaultInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getDefaultInstance"
        + os.linesep
        + "  (JNIEnv *, jobject)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::GetDefaultInstance();"
        + os.linesep
        + "}"
        + os.linesep
        + os.linesep
        + "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv *, jobject)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::CreateInstance();"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    # Handle function declarations properly
    test.add_input(
        "./TestJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    getDefaultInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_getDefaultInstance"
        + os.linesep
        + "  (JNIEnv *, jobject);"
        + os.linesep
        + os.linesep
        + "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_networktables_NetworkTablesJNI"
        + os.linesep
        + " * Method:    createInstance"
        + os.linesep
        + " * Signature: ()I"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT jint JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_networktables_NetworkTablesJNI_createInstance"
        + os.linesep
        + "  (JNIEnv *, jobject)"
        + os.linesep
        + "{"
        + os.linesep
        + "  return nt::CreateInstance();"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_latest_input_as_output(True)

    # Handle functions whose arguments don't have variable names properly
    test.add_input(
        "./DigitalGlitchFilterJNI.cpp",
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI"
        + os.linesep
        + " * Method:    cleanFilter"
        + os.linesep
        + " * Signature: (I)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL Java_edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI_cleanFilter"
        + os.linesep
        + "  (JNIEnv *, jobject, jint)"
        + os.linesep
        + "{"
        + os.linesep
        + "  HAL_CleanFilter(handle);"
        + os.linesep
        + "}"
        + os.linesep,
    )
    test.add_output(
        "/*"
        + os.linesep
        + " * Class:     edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI"
        + os.linesep
        + " * Method:    cleanFilter"
        + os.linesep
        + " * Signature: (I)V"
        + os.linesep
        + " */"
        + os.linesep
        + "JNIEXPORT void JNICALL"
        + os.linesep
        + "Java_edu_wpi_first_wpilibj_hal_DigitalGlitchFilterJNI_cleanFilter"
        + os.linesep
        + "  (JNIEnv *, jobject, jint)"
        + os.linesep
        + "{"
        + os.linesep
        + "  HAL_CleanFilter(handle);"
        + os.linesep
        + "}"
        + os.linesep,
        True,
    )

    test.run(OutputType.FILE)



"""This task runs cpplint.py on a C++ source file.

The `update-cpplint.py` script downloads the latest cpplint.py and edits it for
use by lint.py.

cpplint.py was originally spawned as a subprocess whose output was filtered.
When it was moved into a separate repository, the difference in directories
required it to be used as a module.
"""

import os
import sys

from wpiformat import cpplint
from wpiformat.task import Task


class Lint(Task):
    @staticmethod
    def should_process_file(config_file, name):
        return config_file.is_cpp_file(name)

    @staticmethod
    def run_batch(config_file, names):
        # Handle running in either the root or styleguide directories
        cpplintPrefix = ""
        if os.getcwd().rpartition(os.sep)[2] != "styleguide":
            cpplintPrefix = "styleguide/"

        # Prepare arguments to cpplint.py
        saved_argv = sys.argv

        # Prepare source file and header file regex strings
        srcs = config_file.group("cppSrcFileInclude")
        if srcs:
            srcs = "|".join(srcs)
        else:
            srcs = "a^"
        headers = config_file.group("cHeaderFileInclude") + config_file.group(
            "cppHeaderFileInclude"
        )
        if headers:
            headers = "|".join(headers)
        else:
            headers = "a^"

        sys.argv = ["cpplint.py", "--srcs=" + srcs, "--headers=" + headers] + names

        # Run cpplint.py
        try:
            cpplint.main()
        except SystemExit as e:
            # Restore original arguments
            sys.argv = saved_argv

            # Report success if error code is 0 (False)
            return e.code == False



import os

from .test_tasktest import *
from wpiformat.gtestname import GTestName


def test_gtestname():
    test = TaskTest(GTestName())

    # Ignore TEST macro that isn't for GoogleTest
    test.add_input("./Test.cpp", "AXIS_TEST(Joystick, X)" + os.linesep)
    test.add_latest_input_as_output(True)

    for test_type in ["TEST", "TEST_F", "TEST_P"]:
        # Test suite replacements
        test.add_input("./Test.cpp", f"{test_type}(Foo, Bar) {{}}" + os.linesep)
        test.add_output(f"{test_type}(FooTest, Bar) {{}}" + os.linesep, True)
        test.add_input("./Test.cpp", f"{test_type}(FooTests, Bar) {{}}" + os.linesep)
        test.add_output(f"{test_type}(FooTest, Bar) {{}}" + os.linesep, True)

        # Test case replacements
        test.add_input("./Test.cpp", f"{test_type}(FooTest, BarTest) {{}}" + os.linesep)
        test.add_output(f"{test_type}(FooTest, Bar) {{}}" + os.linesep, True)

        # Test case only named "Test" or "Tests" should generate an error
        test.add_input("./Test.cpp", f"{test_type}(FooTest, Test) {{}}" + os.linesep)
        test.add_latest_input_as_output(False)
        test.add_input("./Test.cpp", f"{test_type}(FooTest, Tests) {{}}" + os.linesep)
        test.add_latest_input_as_output(False)

        # Idempotency
        test.add_input("./Test.cpp", f"{test_type}(FooTest, Bar) {{}}" + os.linesep)
        test.add_latest_input_as_output(True)

        # Duplicate test names to make sure file contents aren't dropped
        test.add_input(
            "./Test.cpp",
            f"{test_type}(FooTest, BarTest) {{}}"
            + os.linesep
            + os.linesep
            + f"{test_type}(FooTest, BarTest) {{}}"
            + os.linesep,
        )
        test.add_output(
            f"{test_type}(FooTest, Bar) {{}}"
            + os.linesep
            + os.linesep
            + f"{test_type}(FooTest, Bar) {{}}"
            + os.linesep,
            True,
        )

        # Test case replacement with args on next line
        test.add_input(
            "./Test.cpp",
            f"{test_type}(" + os.linesep + "    FooTest, BarTest) {{}}" + os.linesep,
        )
        test.add_output(
            f"{test_type}(" + os.linesep + "    FooTest, Bar) {{}}" + os.linesep, True
        )

    # INSTANTIATE_TEST_SUITE_P() test suite replacements
    test.add_input("./Test.cpp", "INSTANTIATE_TEST_SUITE_P(Foo, BarTest," + os.linesep)
    test.add_output("INSTANTIATE_TEST_SUITE_P(FooTests, BarTest," + os.linesep, True)
    test.add_input(
        "./Test.cpp", "INSTANTIATE_TEST_SUITE_P(FooTest, BarTest," + os.linesep
    )
    test.add_output("INSTANTIATE_TEST_SUITE_P(FooTests, BarTest," + os.linesep, True)

    # INSTANTIATE_TEST_SUITE_P() test case replacements
    test.add_input("./Test.cpp", "INSTANTIATE_TEST_SUITE_P(FooTests, Bar," + os.linesep)
    test.add_output("INSTANTIATE_TEST_SUITE_P(FooTests, BarTest," + os.linesep, True)
    test.add_input(
        "./Test.cpp", "INSTANTIATE_TEST_SUITE_P(FooTests, BarTests," + os.linesep
    )
    test.add_output("INSTANTIATE_TEST_SUITE_P(FooTests, BarTest," + os.linesep, True)

    # INSTANTIATE_TEST_SUITE_P() idempotency
    test.add_input(
        "./Test.cpp", "INSTANTIATE_TEST_SUITE_P(FooTests, BarTest," + os.linesep
    )
    test.add_latest_input_as_output(True)

    # INSTANTIATE_TEST_SUITE_P() test case replacement with args on next line
    test.add_input(
        "./Test.cpp",
        "INSTANTIATE_TEST_SUITE_P("
        + os.linesep
        + "    FooTests, BarTests,"
        + os.linesep,
    )
    test.add_output(
        "INSTANTIATE_TEST_SUITE_P("
        + os.linesep
        + "    FooTests, BarTest,"
        + os.linesep,
        True,
    )

    test.run(OutputType.FILE)



import os

from wpiformat.config import Config


def test_config():
    config_file = Config(os.path.abspath(os.getcwd()), ".styleguide")
    assert config_file.is_modifiable_file(
        "." + os.sep + "wpiformat" + os.sep + "javaguidelink.png"
    )
    assert config_file.is_generated_file(
        "." + os.sep + "wpiformat" + os.sep + "wpiformat" + os.sep + "cpplint.py"
    )

    assert not config_file.is_generated_file(
        "." + os.sep + "wpiformat" + os.sep + "diff_cpplint.py"
    )
    assert not config_file.is_generated_file(
        "." + os.sep + "wpiformat" + os.sep + "update_cpplint.py"
    )
    assert not config_file.is_modifiable_file(
        "." + os.sep + "wpiformat" + os.sep + "license.txt"
    )
